! =================================================================================================================================
! MODULE       : stomate_litter
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Update litter and lignine content after litter fall and 
!! calculating litter decomposition.      
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! Modifications from Haicheng Zhang
!! 1) Replaceing the original first-order decomposition model with the microbial explicit model MIMICS, see stomate_litter.f90
!!    stome_som_dynamics.f90
!! 3) Adding a factor of SOM N:P ratio to constrain the BCM of SOP. With higher SOM N:P ratio, BCM rate of SOP is lower.
!! 4) Modifying the equation for calculating the fraction of drainge water compared to total soil water (i.e. f_drain)
!!    see stome_som_dynamics.f90 & stomate_phosphorus.f90
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_litter

  ! modules used:

  USE ioipsl_para
  USE xios_orchidee
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC littercalc,littercalc_clear, deadleaf

  LOGICAL, SAVE                        :: firstcall_litter = .TRUE.       !! first call
!$OMP THREADPRIVATE(firstcall_litter)

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : littercalc_calc
!!
!!\BRIEF        Set the flag ::firstcall_litter to .TRUE. and as such activate section
!! 1.1 of the subroutine littercalc (see below).
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc_clear
    firstcall_litter =.TRUE.
  END SUBROUTINE littercalc_clear


!! ================================================================================================================================
!! SUBROUTINE   : littercalc
!!
!! HZ: A microbial explicit model MIMICS is used here to replace the original CENTURY
!! model to calculate the decomposition of litter and SOM
!!
!!\BRIEF        Calculation of the litter decomposition and therefore of the 
!! heterotrophic respiration from litter following Wieder et al. (2014, BG; 2015,GMD).
!!
!! DESCRIPTION  : The littercal routine splits the litter in 6 pools: 
!! aboveground metabolic, aboveground structural, aboveground woody, 
!! belowground metabolic, belowground structural, belowground woody,  
!! In this version, The microbial explicit model MIMICS (Wieder et al., 2014, 2015)
!! restructured and calibrated by Zhang et al. (2019) is adopted to replace the 
!! previously first-order decomposition model CENTURY 
!!
!! Wieder et al. (2014, BG; 2015,GMD).
!! \latexonly
!! \input{littercalc1.tex}
!! \endlatexonly
!! \n
!! where L is the lignin content of the plant carbon pools considered and CN 
!! its CN ratio. L and CN are fixed parameters for each plant carbon pools,
!! therefore it is the ratio between each plant carbon pool within a PFT, which
!! controlled the part of the total litter, that will be considered as
!! recalcitrant (i.e. structural litter) or labile (i.e. metabolic litter).\n  
!! 
!! The routine calculates the fraction of aboveground litter which is metabolic
!! or structural (the litterpart variable) which is then used in lpj_fire.f90.\n 
!! 
!! In the section 2, the routine calculate the new plant material entering the
!! litter pools by phenological death of plants organs (corresponding to the
!! variable turnover) and by fire, herbivory and others non phenological causes
!! (variable bm_to_litter). This calculation is first done for each PFT and then
!! the values calculated for each PFT are added up. Following the same approach
!! the lignin content of the total structural litter is calculated and will be
!! then used as a factor control of the decomposition of the structural litter
!! (lignin_struc) in the section 5.1.2. A test is performed to avoid that we add
!! more lignin than structural litter. Finally, the variable litterpart is
!! updated.\n
!! 
!! In the section 3 and 4 the temperature and the moisture controlling the
!! decomposition are calculated for above and belowground. For aboveground
!! litter, air temperature and litter moisture are calculated in sechiba and used 
!! directly. For belowground, soil temperature and moisture are also calculated 
!! in sechiba but are modulated as a function of the soil depth. The modulation 
!! is a multiplying factor exponentially distributed between 0 (in depth) and 1
!! in surface.\n  
!! 
!! Then, in the section 5, the routine calculates the structural litter decomposition 
!! (C) following first order kinetics following Parton et al. (1987).
!! \latexonly
!! \input{littercalc2.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter. 
!! k corresponds to
!! \latexonly
!! \input{littercalc3.tex}
!! \endlatexonly
!! \n
!! with littertau the turnover rate, T a function of the temperature and M a function of
!! the moisture described below.\n
!!  
!! Then, the fraction of dead leaves (DL) composed by aboveground structural litter is
!! calculated as following
!! \latexonly
!! \input{littercalc4.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter previously
!! described.\n
!!
!! In the section 5.1, the fraction of decomposed structural litter
!! incorporated to the soil (Input) and its associated heterotrophic respiration are
!! calculated. For structural litter, the C decomposed could go in the active
!! soil carbon pool or in the slow carbon, as described in 
!! stomate_soilcarbon.f90.\n
!! \latexonly
!! \input{littercalc5.tex}
!! \endlatexonly
!! \n
!! with f a parameter describing the fraction of structural litter incorporated
!! into the considered soil carbon pool, C the amount of litter decomposed and L 
!! the amount of lignin in the litter. The litter decomposed which is not
!! incorporated into the soil is respired.\n
!!
!! In the section 5.2, the fraction of decomposed metabolic litter
!! incorporated to the soil and its associated heterotrophic respiration are
!! calculated with the same approaches presented for 5.1 but no control factor
!! depending on the lignin content are used.\n
!! 
!! In the section 6 the dead leaf cover is calculated through a call to the 
!! deadleaf subroutine presented below.\n
!!
!! In the section 7, if the flag SPINUP_ANALYTIC is set to true, we fill MatrixA
!! and VectorB following Lardy(2011).
!!
!! MAIN OUTPUT VARIABLES: ::deadleaf_cover, ::resp_hetero_litter, ::soilcarbon_input, 
!! ::control_temp, ::control_moist
!!
!! REFERENCES:
!! - Parton, WJ, Schimel, DS, Cole, CV, and Ojima, DS. 1987. Analysis
!! of factors controlling soil organic matter levels in Great Plains
!! grasslands. Soil Science Society of America journal (USA)
!! (51):1173-1179.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics(scale=0.5){littercalcflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc (npts, clay, silt, bulk, &
       turnover, bm_to_litter, npp_longterm, &
       veget_cov_max, tsurf, tsoil, soilhum, litterhum, soil_n_min,soil_p_min, &
       tsurf_longterm,tsoil_longterm, litterhum_longterm, soilhum_longterm, litter, som, microbe, &
       microbe_active,socphy_max,litter_avail, litter_not_avail, litter_avail_frac, &								   
       dead_leaves, lignin_struc, lignin_wood, n_mineralisation, p_mineralisation, &
       deadleaf_cover, resp_hetero_litter, som_input, control_temp, &
       control_moist,control_tempvmax, control_tempkm, control_tempdesorp, &
       MatrixA, VectorB, CN_target,CN_mictarget, CN_som_litter_longterm, tau_CN_longterm, &
       CP_target,CP_mictarget, CP_som_litter_longterm, &
       sla_calc,do_slow)
    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                  :: npts               !! Domain size - number of grid pixels
	REAL(r_std), DIMENSION(npts), INTENT(in)              		:: clay               !! Clay fraction (unitless, 0-1) 
    REAL(r_std), DIMENSION(npts), INTENT(in)              		:: silt               !! Silt fraction (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: turnover         !! Turnover rates of plant biomass 
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: bm_to_litter     !! Conversion of biomass to litter 
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                  :: veget_cov_max      !! PFT "Maximal" coverage fraction of a PFT 
                                                                                      !! defined in the input vegetation map 
                                                                                      !! @tex $(m^2 m^{-2})$ @endtex 
	REAL(r_std), DIMENSION(npts,nvm), INTENT(in)            	:: npp_longterm       !! "Long term" mean yearly primary 
                                                                                      !! productivity 
	REAL(r_std), DIMENSION(npts), INTENT(in)                    :: tsurf              !! Temperature (K) at the surface
	REAL(r_std), DIMENSION(npts), INTENT(in)                    :: tsurf_longterm       !! long-term land surface Temperature (K) at 2 m high
	REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: tsoil              !! Soil temperature (K)
	REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: tsoil_longterm       !! long-term soil Temperature (K) at 2 m high
	REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: soilhum_longterm   !! long-term avarage Soil humidity (unitless)
    REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: soilhum            !! Daily soil humidity of each soil layer 
                                                                                      !! (unitless)
	REAL(r_std), DIMENSION(npts), INTENT(in)               		:: litterhum_longterm !! long-term avarage litter humidity (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: litterhum          !! Daily litter humidity (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nnspec),INTENT(in)          :: soil_n_min         !! mineral nitrogen in the soil (gN/m**2)  
                                                                                      !! (first index=npts, second index=nvm, third index=nnspec)
	REAL(r_std), DIMENSION(npts,nvm,npspec),INTENT(in)          :: soil_p_min         !! mineral P in the soil (gP/m**2)  
                                                                                      !! (first index=npts, second index=nvm, third index=npspec) 											   
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(in)          :: litter_not_avail   !! litter not edible for animal
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                  :: sla_calc           !! leaf age-related SLA
    LOGICAL,INTENT(in)                                          :: do_slow            !! called only once per day
	REAL(r_std),DIMENSION(npts),INTENT(in)      				:: bulk             !! Bulk density (kg/m**3)

    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts), INTENT(out)                   :: deadleaf_cover     !! Fraction of soil covered by dead leaves 
                                                                                      !! over all PFTs (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)               :: resp_hetero_litter !! Litter heterotrophic respiration. The unit
                                                                                      !! is given by m^2 of ground. g C m^{-2} dt^-1 
                                                                                      !! @tex $(gC dt_sechiba one\_day^{-1}) m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(out) :: som_input        !! Quantity of Carbon (or Nitrogen) going into SOM pools
                                                                                      !! from litter decomposition. The unit is  
                                                                                      !! given by m^2 of ground 
                                                                                      !! @tex $(gC(orN) m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_temp       !! Temperature control of heterotrophic 
                                                                                      !! respiration, above and below (0-1, 
                                                                                      !! unitless)
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_moist      !! Moisture control of heterotrophic 
	REAL(r_std), DIMENSION(npts,nlevs) , INTENT(out)            :: control_tempvmax   !! Temperature control of heterotrophic 
                                                                                      !! respiration, above and below (0-1,unitless)
	REAL(r_std), DIMENSION(npts,nlitt,nlevs), INTENT(out)       :: control_tempkm     !! Temperature control of heterotrophic 
                                                                                      !! respiration, above and below (0-1,unitless)
                                                                                      !! respiration (0.25-1, unitless)
	REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)       		:: control_tempdesorp !! Temperature control of SOC desorption (0-1,unitless)
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(out) :: MatrixA          !! Matrix containing the fluxes between the
                                                                                      !! carbon pools per sechiba time step 
                                                                                      !! @tex $(mgC.cm^3.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)         :: VectorB          !! Vector containing the litter increase per
                                                                                      !! sechiba time step
                                                                                      !! @tex $(mgC cm^{-3})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncarb), INTENT(out)           :: CN_target        !! C to N ratio of SOM flux from one pool to another (gC gN-1)
    REAL(r_std), DIMENSION(npts,nvm,ncarb), INTENT(out)           :: CP_target        !! C to P ratio of SOM flux from one pool to another (gC gP-1)
	REAL(r_std), DIMENSION(npts,nvm,nmic), INTENT(out)      :: CN_mictarget       !! C to N ratio of microbes (gC gN-1)
    REAL(r_std), DIMENSION(npts,nvm,nmic), INTENT(out)      :: CP_mictarget      !! C to P ratio of microbes (gC gP-1)
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(out)           :: litter_avail_frac!! litter fraction that is edible for animal
	REAL(r_std), DIMENSION(npts,nvm),INTENT(out)  		  		  :: socphy_max       !! Concentration of maximum physically-protected SOM pools
																			          !! The unit is given by cm^3 ofsoil @tex $(mgC cm^{-3})$ @endtex																																 
    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout) :: litter   !! Metabolic and structural litter,above and
                                                                                      !! below ground. The unit is given by m^2 of 
                                                                                      !! ground @tex $(gC m^{-2})$ @endtex
	REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(in) 		 :: som       !! SOM pools: somp, somc, or soma, \f$(gC m^{2})$\f
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs,nelements), INTENT(inout) :: microbe   !! Stock of microbial biomass-C
                                                                                      !! The unit is given by m^2 of
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs,nelements), INTENT(inout) :: microbe_active     	!! Concentration of active microbial biomass.
																									!! The unit is given by m^3 of 
																									!! soil @tex $(mgC cm^{-3})$ @endtex
																					  !! ground @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(inout)       :: litter_avail       !! edible litter for animal 
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)       :: dead_leaves        !! Dead leaves per ground unit area, per PFT,
                                                                                      !! metabolic and structural in 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_struc       !! Ratio Lignin content in structural litter,
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_wood        !! Ratio Lignin/Carbon in woody litter,	
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)             :: n_mineralisation   !! mineralisation of N (gN m-2 timestep-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)             :: p_mineralisation   !! mineralisation of P (gP m-2 timestep-1)

    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(inout)     :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), INTENT(inout)                                  :: tau_CN_longterm        !! Counter used for calculating the longterm CN_ratio of som and litter pools [seconds]
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(inout)     :: CP_som_litter_longterm !! Longterm NP ratio of litter and som pools (gN/gP)

    !! 0.4 Local variables
 
    REAL(r_std)                                                 :: dt                 !! Number of sechiba(fast processes) time-step per day 
    REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:)          :: litterfrac         !! The fraction of leaves, wood, etc. that 
                                                                                      !! goes into metabolic and structural 
                                                                                      !! litterpools (0-1, unitless)
!$OMP THREADPRIVATE(litterfrac)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)                :: z_soil             !! Soil levels (m)
!$OMP THREADPRIVATE(z_soil)
    REAL(r_std), DIMENSION(npts)                                :: rpc                !! Integration constant for vertical root 
                                                                                      !! profiles (unitless)
	REAL(r_std), DIMENSION(npts,nvm,nmic,nlevs)    				:: CN_microbe       !! C to N ratio of microbes (gC gN-1)
    REAL(r_std), DIMENSION(npts,nvm,nmic,nlevs)      			:: CP_microbe       !! C to P ratio of microbes (gC gP-1)
	REAL(r_std), SAVE, DIMENSION(nlitt,ncarb,nlevs)             :: frac_litsom        !! Fraction of litter that goes directly into SOM pools, bypassing
	REAL(r_std), SAVE, DIMENSION(nlitt,nmic,nlevs)              :: vmax_modlitt       !! Tuning coefficient of the maximum reaction velocity (Vmax)
	REAL(r_std), SAVE, DIMENSION(nlitt,nmic,nlevs)              :: khalf_modlitt      !! Tuning coefficient of half-saturation constant (Km)
                                                                                      !! litter and microbial pools
	REAL(r_std), SAVE, DIMENSION(nlitt,nmic,nlevs)              :: frac_soil          !! Fixed fraction of litter that goes into microbial biomass pool.
                                                                                      !! The remaining part goes to the atmosphere
	REAL(r_std), DIMENSION(npts,nvm,nlitt,nmic,nlevs)     		:: frac_litt_cue      !! Dynamic fraction of litter that goes into microbial biomass pool.
                                                                                      !! The remaining part goes to the atmosphere
	REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements)		:: litter_density     !! Metabolic and structural litter,above and
																					  !! below ground. The unit is given by cm^3 of 
																					  !! soil @tex $(mgC cm^{-3})$ @endtex
	REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements)	  		:: som_density        !! Concentration of SOM pools
																					  !! below ground. The unit is given by cm^3 of 
																					  !! soil @tex $(mgC cm^{-3})$ @endtex
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs,nelements)		:: microbe_density    !! Concentration of microbial biomass.
																					  !! The unit is given by m^3 of 
																					  !! soil @tex $(mgC cm^{-3})$ @endtex
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs,nelements)		:: microbe_dormant    !! Concentration of dormant microbial biomass.
																					  !! The unit is given by m^3 of 
																					  !! soil @tex $(mgC cm^{-3})$ @endtex
	REAL(r_std), DIMENSION(npts,nvm,nmic,ncarb,nlevs)     		:: frac_micsom        !! Fraction of microbial biomass turnover that goes into 
                                                                                      !! SOM pools
	REAL(r_std), DIMENSION(nlitt,nmic,nlevs)              		:: vmax_mic       	  !! Tuning coefficient of the maximum reaction velocity (Vmax)
	REAL(r_std), DIMENSION(nlitt,nmic,nlevs)              		:: khalf_mic      	  !! Tuning coefficient of half-saturation constant (Km)
	REAL(r_std), DIMENSION(npts,nvm)             				:: depth_litter       !! Depth of the litter layer (cm)
	REAL(r_std), DIMENSION(npts,nvm)             				:: bd_litter       	  !! bulk density of litter layer (cm)
	REAL(r_std), DIMENSION(npts,nvm)                			:: mass_above         !! total aboveground organic matters (g cm^{-2})
	REAL(r_std), DIMENSION(npts,nvm)                			:: litt_decay         !! total litter decomposition rate (g C m^{-2} dt_stomate-1)
	REAL(r_std), DIMENSION(npts,nvm)                			:: cue_ave            !! Average Carbon Use Efficiency of microbes for each PFT																													  
	REAL(r_std), DIMENSION(npts,nvm,nlevs)             			:: frac_metabave      !! average fraction of metabolic litter (0-1,unitless)
	REAL(r_std), DIMENSION(npts,nvm,nmic,nlevs)           		:: tau_mic	      	  !! Microbial turnover rate
	REAL(r_std), DIMENSION(npts,nvm,nmic,nlevs,nelements) 		:: kk_mic	      	  !! Microbial turnover rate
	REAL(r_std), DIMENSION(npts)                                :: vmax_lignin        !! Controlling of litter lignin content on microbial reaction velocity
	!$OMP THREADPRIVATE(frac_soil)
    REAL(r_std), DIMENSION(npts)                                :: tsoil_decomp       !! Temperature used for decompostition in 
                                                                                      !! soil (K)
	REAL(r_std), DIMENSION(npts)                                :: tsoil_decomplong   !! Longterm soil temperature used for decompostition in 
                                                                                      !! soil (K) 
    REAL(r_std), DIMENSION(npts)                                :: soilhum_decomp     !! Humidity used for decompostition in soil
                                                                                      !! (unitless)
	REAL(r_std), DIMENSION(npts)                                :: soilhumlong_decomp !! Long term averaged humidity used for decompostition in soil
                                                                                      !! (unitless)
	REAL(r_std), DIMENSION(npts,nlevs)            				:: control_temptau    !! Temperature control of active microbes  : HZ added
	REAL(r_std), DIMENSION(npts,nlevs)            				:: control_tempmictau !! Temperature control of microbial turnover rate : HZ added
	REAL(r_std), DIMENSION(npts,nlevs)             				:: control_moistau    !! Moisture control of active microbe : HZ added
	REAL(r_std), DIMENSION(npts,nlevs)                          :: control_moistvmax  !! Moisture control of Vmax
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs)					:: frac_micactive     !! Fraction of active microbes  : HZ added
																					  !! soil @tex $(mgC cm^{-3} / dt)$ @endtex
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs)					:: cflux_litt2mic     !! Flux of CNP from litter pool to microbial pool
																					  !! The unit is given by cm^3 of 
																					  !! soil @tex $(mgC cm^{-3} / dt)$ @endtex
    REAL(r_std), DIMENSION(npts,nelements)                      :: qd                 !! Quantity of structural or metabolic litter
                                                                                      !! decomposed @tex $(mgC cm^{-3})$ @endtex
	REAL(r_std), DIMENSION(npts,nmic,nelements)                 :: qd_mic             !! Quantity of structural, wood or metabolic litter
                                                                                      !! decomposed by different microbial types
																					  !! @tex $(mg C cm^{-3})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_struc          !! Old structural litter, above and below 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_woody          !! Old woody litter, above and below 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements)      :: litter_inc         !! Increase of metabolic and structural 
                                                                                      !! litter, above and below ground. The unit 
                                                                                      !! is given by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
	REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs)    		  	:: cflux_litt         !! decomposition rate of each litt pool \f$(gC m^{2}hr-1)$\f
	REAL(r_std), DIMENSION(npts,nmic,nvm,nlevs)    		  	    :: cflux_mic          !! decomposition rate of each microbe pool \f$(gC m^{2}hr-1)$\f
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_struc_inc   !! Lignin increase in structural litter, 
                                                                                      !! above and below ground. The unit is given 
                                                                                      !! by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex                                             
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_wood_inc    !! Lignin increase in woody litter, 
                                                                                      !! above and below ground. The unit is given 
                                                                                      !! by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                                :: zdiff_min          !! Intermediate field for looking for minimum
                                                                                      !! of what? this is not used in the code. 
                                                                                      !! [??CHECK] could we delete it?
	REAL(r_std)                              				  	:: fdecomp_np         !! Fraction of potentially decomposed litter
	REAL(r_std)                                					:: np_supply          !! N or P availiable for microbe (litter decay+NPmineral)
	REAL(r_std)                                					:: np_need            !! Potential N or P requirement for microbe
    CHARACTER(LEN=10), DIMENSION(nlitt)                         :: litter_str         !! Messages to write output information about
                                                                                      !! the litter
    CHARACTER(LEN=22), DIMENSION(nparts)                        :: part_str           !! Messages to write output information about
                                                                                      !! the plant
    CHARACTER(LEN=7), DIMENSION(ncarb)                          :: carbon_str         !! Messages to write output information about
                                                                                      !! the soil carbon
    CHARACTER(LEN=5), DIMENSION(nlevs)                          :: level_str          !! Messages to write output information about
                                                                                      !! the level (aboveground or belowground litter)
    INTEGER(i_std)                                              :: i,j,k,l,m,iem,ig   !! Indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_soil_n_min       !! soil_n_min function response used for defing C to N target ratios (-) 
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_pnc              !! plant nitrogen concentration function response used for defining C to N target ratios (-)  
    REAL(r_std), DIMENSION(npts,nvm)                            :: soilN_density      !! Density of soil mineral N (mg N cm-3)
	REAL(r_std), DIMENSION(npts,nvm)                            :: soilP_density      !! Density of soil mineral P (mg P cm-3)
	REAL(r_std), DIMENSION(npts,nvm)                            :: npp_longtermave    !! long-term average NPP
	INTEGER(i_std)                                              :: itarget            !! target som pool 
    REAL(r_std), DIMENSION(npts,nvm,nparts)                     :: CN                 !! CN ratio of the litter pools 
    REAL(r_std), DIMENSION(npts,nelements)                      :: summ 
    INTEGER(i_std)                                              :: ier                !! Error handling
!_ ================================================================================================================================

    IF (printlev>=3) WRITE(numout,*) 'Entering littercalc'

  !! 1. Initialisations of the different fields during the first call of the routine 
    dt = dt_sechiba/one_day
    IF ( firstcall_litter ) THEN

       IF ( .NOT. ALLOCATED(litterfrac) ) THEN
          ALLOCATE(litterfrac(npts,nvm,nparts,nlitt), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3, 'littercalc', 'ALLOCATE litterfrac', &
                    'Not enough memory', 'npts * nvm * nparts * nlitt')
       ENDIF
	   
       !!1.1.1 Fraction of litter that goes directly into SOM pools, bypassing litter and microbial pools
	   frac_litsom(:,:,:) = zero
	   frac_litsom(imetabolic,isomp,iabove) = frac_litmet_somp_ab
	   frac_litsom(istructural,isomc,iabove) = frac_litstr_somc_ab
	   frac_litsom(iwoody,isomc,iabove) = frac_litwood_somc_ab
	   frac_litsom(imetabolic,isomp,ibelow) = frac_litmet_somp_be
	   frac_litsom(istructural,isomc,ibelow) = frac_litstr_somc_be
	   frac_litsom(iwoody,isomc,ibelow) = frac_litwood_somc_be

       !! 1.1.2 Coefficients of the maximum reaction velocity (Vmax) and half-saturation constant (Km)
	   vmax_modlitt(:,:,:)=zero
       vmax_modlitt(imetabolic,imicr,:) = 10.0*av_vmax_litt           ! 10.0
	   vmax_modlitt(istructural:iwoody,imicr,:) = 2.0*av_vmax_litt    ! 2.0
	   vmax_modlitt(imetabolic,imick,:) = 3.0*av_vmax_litt            ! 3.0
	   vmax_modlitt(istructural:iwoody,imick,:) = 3.0 *av_vmax_litt   ! 3.0
	   khalf_modlitt(:,:,:)=zero
	   khalf_modlitt(imetabolic,imicr,:) = 8.0          ! 8.0 --->10
	   khalf_modlitt(istructural:iwoody,imicr,:) = 2.0   ! 2.0
	   khalf_modlitt(imetabolic,imick,:) = 2.0           ! 2.0
	   khalf_modlitt(istructural:iwoody,imick,:) = 4.0   ! 4.0
      
       !! 1.1.3 Carbon use efficiency of microbes when consuming different substrates
       !       (litter, availiable SOM pool -> micr, mick)
       !       1-frac_soil goes into atmosphere
	   !       Default fixed values
       frac_soil(:,:,:) = zero

       frac_soil(imetabolic,imicr,iabove) = frac_litt_metab_ra
	   frac_soil(istructural,imicr,iabove) = frac_litt_struct_ra
	   frac_soil(iwoody,imicr,iabove) = frac_litt_woody_ra
	   frac_soil(imetabolic,imick,iabove) = frac_litt_metab_ka
	   frac_soil(istructural,imick,iabove) = frac_litt_struct_ka
	   frac_soil(iwoody,imick,iabove) = frac_litt_woody_ka
	   
       frac_soil(imetabolic,imicr,ibelow) = frac_litt_metab_rb
	   frac_soil(istructural,imicr,ibelow) = frac_litt_struct_rb
	   frac_soil(iwoody,imicr,ibelow) = frac_litt_woody_rb
	   frac_soil(imetabolic,imick,ibelow) = frac_litt_metab_kb
	   frac_soil(istructural,imick,ibelow) = frac_litt_struct_kb
	   frac_soil(iwoody,imick,ibelow) = frac_litt_woody_kb
       !! 1.1.4  Stoichiometery of microbial biomass and 
	   !         initialization of the microbial biomass
	   DO m=1,nvm
		   WHERE (microbe(:,imicr,m,:,initrogen).GT.zero)
		      CN_microbe(:,m,imicr,:) = microbe(:,imicr,m,:,icarbon)/microbe(:,imicr,m,:,initrogen)
		   ELSEWHERE
		      CN_microbe(:,m,imicr,:) = cn_micr(m)
		   ENDWHERE
		   !
		   WHERE (microbe(:,imick,m,:,initrogen).GT.zero)
		      CN_microbe(:,m,imick,:) = microbe(:,imick,m,:,icarbon)/microbe(:,imick,m,:,initrogen)
		   ELSEWHERE
		      CN_microbe(:,m,imick,:) = cn_micr(m)
		   ENDWHERE
		   !
		   WHERE (microbe(:,imicr,m,:,iphosphorus).GT.zero)
		      CP_microbe(:,m,imicr,:) = microbe(:,imicr,m,:,icarbon)/microbe(:,imicr,m,:,iphosphorus)
		   ELSEWHERE
		      CP_microbe(:,m,imicr,:) = cp_micr(m)
		   ENDWHERE
		   !
		   WHERE (microbe(:,imick,m,:,iphosphorus).GT.zero)
		      CP_microbe(:,m,imick,:) = microbe(:,imick,m,:,icarbon)/microbe(:,imick,m,:,iphosphorus)
		   ELSEWHERE
		      CP_microbe(:,m,imick,:) = cp_micr(m)
		   ENDWHERE
	   ENDDO
	   
	!   WRITE(numout,*) 'First_call_NVM1',nvm
	!   WRITE(numout,*) 'cn_micr:',cn_micr(:)
	!   WRITE(numout,*) 'cn_mick:',cn_mick(:)
	!   WRITE(numout,*) 'cp_micr:',cp_micr(:)
	!   WRITE(numout,*) 'cp_mick:',cp_mick(:)
	!  WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,icarbon)
	!   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,icarbon)
	!   WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,initrogen)
	!   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,initrogen)
	!   WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,iphosphorus)
	!   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,iphosphorus)
	!   !WRITE(numout,*) 'ZHC0_stomate_mick_ab:',microbe(1,imick,:,iabove,icarbon)
	!   !WRITE(numout,*) 'ZHC0_stomate_micr_be:',microbe(1,imicr,:,ibelow,icarbon)
	!   !WRITE(numout,*) 'ZHC0_stomate_mick_be:',microbe(1,imicr,:,ibelow,icarbon)
	   WHERE (microbe(:,:,:,:,icarbon).LE.zero)
			microbe(:,:,:,:,icarbon) = 0.0
			microbe(:,:,:,:,initrogen) = 0.0
			microbe(:,:,:,:,iphosphorus) = 0.0
	   ENDWHERE
!	   WRITE(numout,*) 'ZHC0.1_stomate_micr_ab:',microbe(1,imicr,:,iabove,icarbon)
!	   WRITE(numout,*) 'First_call_NVM2',nvm
!	   WRITE(numout,*) 'cn_micr:',cn_micr(:)
!	   WRITE(numout,*) 'cn_mick:',cn_mick(:)
!	   WRITE(numout,*) 'cp_micr:',cp_micr(:)
!	   WRITE(numout,*) 'cp_mick:',cp_mick(:)
!	   WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,icarbon)
!	   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,icarbon)
!	   WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,initrogen)
!	   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,initrogen)
!	   WRITE (numout,*) 'Microbes_ab', microbe(1,:,7,iabove,iphosphorus)
!	   WRITE (numout,*) 'Microbes_be', microbe(1,:,7,ibelow,iphosphorus)	

       !! 1.2 soil levels
       ALLOCATE(z_soil(0:nslm), stat=ier)
       IF ( ier /= 0 ) CALL ipslerr_p(3,'littercalc','Pb in allocate of z_soil','','')

       z_soil(0) = zero
       z_soil(1:nslm) = diaglev(1:nslm)

       
       !! 1.3 messages
       litter_str(imetabolic) = 'metabolic'
       litter_str(istructural) = 'structural'
       litter_str(iwoody) = 'woody'

       carbon_str(isomp) = 'SOMp physically protected SOM pool'
       carbon_str(isomc) = 'SOMc chemically protected SOM pool'
       carbon_str(isoma) = 'SOMa availiable SOM pool'

       level_str(iabove) = 'above'
       level_str(ibelow) = 'below'

       part_str(ileaf) = 'leaves'
       part_str(isapabove) = 'sap above ground'
       part_str(isapbelow) = 'sap below ground'
       part_str(iheartabove) = 'heartwood above ground'
       part_str(iheartbelow) = 'heartwood below ground'
       part_str(iroot) = 'roots'
       part_str(ifruit) = 'fruits'
       part_str(icarbres) = 'carbohydrate reserve'
       part_str(ilabile) = 'labile reserve'

       IF (printlev >=2) THEN
          WRITE(numout,*) 'litter:'

          WRITE(numout,*) '   > C/N ratios: '
          DO k = 1, nparts
             WRITE(numout,*) '       ', part_str(k), ': ',CN_fix(k)
          ENDDO

          WRITE(numout,*) '   > Lignine/C ratios: '
          DO k = 1, nparts
             WRITE(numout,*) '       ', part_str(k), ': ',LC(:,k)
          ENDDO

          WRITE(numout,*) '   > fraction of compartment that goes into litter: '
          ALLOCATE(litterfrac(npts,nvm,nparts,nlitt), stat=ier)
            DO k = 1, nparts
              DO m = 1, nlitt
                WRITE(numout,*) '       ', part_str(k), '-> ',litter_str(m), ':',litterfrac(:,:,k,m)
              ENDDO
          ENDDO

          WRITE(numout,*) '   > scaling depth for decomposition (m): ',z_decomp

          WRITE(numout,*) '   > litter decomposition flux fraction that really goes '
          WRITE(numout,*) '     into carbon pools (rest into the atmosphere):'
  !DSG: wrong iteration over frac_soil
  ! needs to be fixed
  !        DO m = 1, nlitt
  !           DO l = 1, nlevs
  !              DO k = 1, ncarb
  !                 WRITE(numout,*) '       ',litter_str(m),' ',level_str(l),' -> ',&
  !                      carbon_str(k),':', frac_soil(m,k,l)
  !              ENDDO
  !           ENDDO
  !        ENDDO
       END IF ! printlev >=2
       firstcall_litter = .FALSE.

    ENDIF

	!WRITE(numout,*) 'SecondCall', nvm
	!WRITE(numout,*) 'ZHC0_stomate_micr_ab:',microbe(1,imicr,:,iabove,icarbon)
	!WRITE(numout,*) 'ZHC0_stomate_mick_ab:',microbe(1,imick,:,iabove,icarbon)
	!WRITE(numout,*) 'ZHC0_stomate_micr_be:',microbe(1,imicr,:,ibelow,icarbon)
	!WRITE(numout,*) 'ZHC0_stomate_mick_be:',microbe(1,imicr,:,ibelow,icarbon)

    !! 1.1.3 litter fractions:
    !!   what fraction of leaves, wood, etc. goes into metabolic and structural litterpools
	!
	! Microbial C:N and C:P ratios are assumed to be fixed values
    ! PFT 1 needs to be initialised for when everything is printed below
    litterfrac(:,1,:,:) = zero
    CN(:,:,:)           = zero
	
	!WRITE (numout,*) 'ZHC0_stomate_litter: litterfrac'
    DO k = 1, nparts
       WHERE((bm_to_litter(:,:,k,initrogen)+turnover(:,:,k,initrogen)).GT.min_stomate) 
          CN(:,:,k) = (bm_to_litter(:,:,k,icarbon)+turnover(:,:,k,icarbon))/ & 
               (bm_to_litter(:,:,k,initrogen)+turnover(:,:,k,initrogen))
       ELSEWHERE
          CN(:,:,k) = CN_fix(k)
       ENDWHERE
       DO j = 2,nvm

          IF ( ((k == isapabove) .OR. (k == isapbelow) .OR. (k == iheartabove) .OR. (k == iheartbelow)) &
               .AND. is_tree(j) ) THEN
             litterfrac(:,j,k,iwoody) = 1.
             litterfrac(:,j,k,imetabolic) = zero
             litterfrac(:,j,k,istructural) = zero
          ELSE 
             IF( (k == icarbres) .OR. (k == ilabile)) THEN
                litterfrac(:,j,k,imetabolic) = 1.0
             ELSE
                litterfrac(:,j,k,imetabolic) = MAX(metabolic_ref_frac - metabolic_LN_ratio  * LC(j,k) * CN(:,j,k),zero)
             ENDIF
             litterfrac(:,j,k,istructural) = 1. - litterfrac(:,j,k,imetabolic)
             litterfrac(:,j,k,iwoody) = 0.0
             
          ENDIF
          
       END DO
    !   WRITE (numout,*) 'ipart', k,'fraction_met_stru_wood',litterfrac(1,7,k,:)
    END DO

    CN_target(:,:,:)=0.0

    ! C to N target ratios of differnt pools
    f_soil_n_min = MIN((soil_n_min(:,:,iammonium) + soil_n_min(:,:,initrate)), 1.5)

    !DSG no flexibility
    f_soil_n_min(:,:) = SPREAD(fixed_fsoiln,NCOPIES=npts,DIM=1)

	!
	! H.Zhang(2019.06.28): C:N and N:P ratios for different SOM pools are set based on 
	! C:N and N:P ratios of CENTURY pools. SOMa = active, SOMc = slow, SOMp = passive
    ! CN ratio ranges between 3 and 15 for availiable pool (active pool)
    CN_target(:,:,isoma)= CN_target_isoma_ref + CN_target_isoma_Nmin * (f_soil_n_min(:,:)-un)
    ! CN ratio ranges between 12 and 20 for chemically recalcitrant pool (slow pool)
    CN_target(:,:,isomc)= CN_target_isomc_ref + CN_target_isomc_Nmin * (f_soil_n_min(:,:)-un)
    ! CN ratio ranges between 7 and 10 for physically protected pool (passive pool)
    CN_target(:,:,isomp)= CN_target_isomp_ref + CN_target_isomp_Nmin * (f_soil_n_min(:,:)-un)
	!
	! 1.1.4. CP stoichiometry 
    ! CURRENT STATUS: CP ratio is not flexible
    CP_target(:,:,:)=0.0
    ! C to P target ratios of different SOM pools
    ! we keep the target NP ratio of SOM fixed (biochemical mineralization will cause the actual NP ratio to vary):
    CP_target(:,:,isoma)  = CN_target(:,:,isoma) * NP_target_isoma_ref
    CP_target(:,:,isomc)  = CN_target(:,:,isomc) * NP_target_isomc_ref 
    CP_target(:,:,isomp)  = CN_target(:,:,isomp) * NP_target_isomp_ref 
	!
	! HZ: For Microbes
	DO m=1,nvm
		WHERE (microbe(:,imicr,m,:,initrogen).GT.zero)
			CN_microbe(:,m,imicr,:) = microbe(:,imicr,m,:,icarbon)/microbe(:,imicr,m,:,initrogen)
		ELSEWHERE
			CN_microbe(:,m,imicr,:) = cn_micr(m)
		ENDWHERE
		   !
		WHERE (microbe(:,imick,m,:,initrogen).GT.zero)
			CN_microbe(:,m,imick,:) = microbe(:,imick,m,:,icarbon)/microbe(:,imick,m,:,initrogen)
		ELSEWHERE
			CN_microbe(:,m,imick,:) = cn_micr(m)
		ENDWHERE
		   !
		WHERE (microbe(:,imicr,m,:,iphosphorus).GT.zero)
			CP_microbe(:,m,imicr,:) = microbe(:,imicr,m,:,icarbon)/microbe(:,imicr,m,:,iphosphorus)
		ELSEWHERE
			CP_microbe(:,m,imicr,:) = cp_micr(m)
		ENDWHERE
		   !
		WHERE (microbe(:,imick,m,:,iphosphorus).GT.zero)
			CP_microbe(:,m,imick,:) = microbe(:,imick,m,:,icarbon)/microbe(:,imick,m,:,iphosphorus)
		ELSEWHERE
			CP_microbe(:,m,imick,:) = cp_micr(m)
		ENDWHERE
		!
		CN_mictarget(:,m,imicr) = cn_micr(m)+ CN_target_micr_Nmin * (f_soil_n_min(:,m)-un)
		CN_mictarget(:,m,imick) = cn_mick(m)+ CN_target_mick_Nmin * (f_soil_n_min(:,m)-un)
		CP_mictarget(:,m,imicr) = cp_micr(m)/cn_micr(m)*CN_mictarget(:,m,imicr)
		CP_mictarget(:,m,imick) = cp_mick(m)/cn_mick(m)*CN_mictarget(:,m,imick)
	ENDDO
    
	!WRITE (numout,*) 'ZHC0000_CN',CN_target(1,7,:)
	!WRITE (numout,*) 'ZHC0000_CN',f_soil_n_min(1,:)
    !! 1.4 set output to zero
    deadleaf_cover(:)       = zero
    resp_hetero_litter(:,:) = zero
	litt_decay(:,:) = zero
	cue_ave(:,:) = zero				   				
    som_input(:,:,:,:)      = zero

    
  !! 2. Add biomass to different litterpools (per m^2 of ground)
    
    !! 2.1 first, save old structural litter (needed for lignin fractions).
    !     above/below
    DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO m = 1,nvm !Loop over PFTs
          old_struc(:,m,l) = litter(:,istructural,m,l,icarbon)
          old_woody(:,m,l) = litter(:,iwoody,m,l,icarbon)
       ENDDO
    ENDDO

    
    !! 2.2 update litter, dead leaves, and lignin content in structural litter
    litter_inc(:,:,:,:,:) = zero
    lignin_struc_inc(:,:,:) = zero
    lignin_wood_inc(:,:,:) = zero
	
    DO j = 1,nvm !Loop over PFTs

       !! 2.2.1 litter
       DO k = 1, nlitt    !Loop over litter pools (metabolic and structural)

          DO l = 1, nelements  ! Loop over element pools (carbon and nitrogen) 
          !! 2.2.2 calculate litter increase (per m^2 of ground).
          !       Only a given fracion of fruit turnover is directly coverted into litter.
          !       Litter increase for each PFT, structural and metabolic, above/below
          litter_inc(:,k,j,iabove,l) =                                           &
               litterfrac(:,j,ileaf      ,k) * bm_to_litter(:,j,ileaf      ,l) + &
               litterfrac(:,j,isapabove  ,k) * bm_to_litter(:,j,isapabove  ,l) + &
               litterfrac(:,j,iheartabove,k) * bm_to_litter(:,j,iheartabove,l) + &
               litterfrac(:,j,ifruit     ,k) * bm_to_litter(:,j,ifruit     ,l) + &
               litterfrac(:,j,icarbres   ,k) * bm_to_litter(:,j,icarbres   ,l) + &
               litterfrac(:,j,ilabile    ,k) * bm_to_litter(:,j,ilabile    ,l) + &
               litterfrac(:,j,ileaf      ,k) * turnover(:,j,ileaf      ,l)     + &
               litterfrac(:,j,isapabove  ,k) * turnover(:,j,isapabove  ,l)     + &
               litterfrac(:,j,iheartabove,k) * turnover(:,j,iheartabove,l)     + &
               litterfrac(:,j,ifruit     ,k) * turnover(:,j,ifruit     ,l)     + &
               litterfrac(:,j,icarbres   ,k) * turnover(:,j,icarbres   ,l)     + & 
               litterfrac(:,j,ilabile    ,k) * turnover(:,j,ilabile    ,l)

          litter_inc(:,k,j,ibelow,l) = &
               litterfrac(:,j,isapbelow,k)   * bm_to_litter(:,j,isapbelow  ,l) + &
               litterfrac(:,j,iheartbelow,k) * bm_to_litter(:,j,iheartbelow,l) + &
               litterfrac(:,j,iroot,k)       * bm_to_litter(:,j,iroot      ,l) + &
               litterfrac(:,j,isapbelow,k)   * turnover(:,j,isapbelow  ,l)     + &
               litterfrac(:,j,iheartbelow,k) * turnover(:,j,iheartbelow,l)     + &
               litterfrac(:,j,iroot,k)       * turnover(:,j,iroot      ,l)    
             
          ENDDO
          !! 2.2.3 dead leaves, for soil cover.
          dead_leaves(:,j,k) = &
               dead_leaves(:,j,k) + &
               litterfrac(:,j,ileaf,k) * ( bm_to_litter(:,j,ileaf,icarbon) + turnover(:,j,ileaf,icarbon) )
       ENDDO
       
       !! 2.2.4 lignin increase in structural litter

       lignin_struc_inc(:,j,iabove) = &
            LC(j,ileaf)       * bm_to_litter(:,j,ileaf,icarbon) + &
            LC(j,isapabove)   * bm_to_litter(:,j,isapabove,icarbon) + &
            LC(j,iheartabove) * bm_to_litter(:,j,iheartabove,icarbon) + &
            LC(j,ifruit)      * bm_to_litter(:,j,ifruit,icarbon) + &
            LC(j,icarbres)    * bm_to_litter(:,j,icarbres,icarbon) + &
            LC(j,ilabile)     * bm_to_litter(:,j,ilabile,icarbon) + &
            LC(j,ileaf)       * turnover(:,j,ileaf,icarbon) + &
            LC(j,isapabove)   * turnover(:,j,isapabove,icarbon) + &
            LC(j,iheartabove) * turnover(:,j,iheartabove,icarbon) + &
            LC(j,ifruit)      * turnover(:,j,ifruit,icarbon) + &
            LC(j,icarbres)    * turnover(:,j,icarbres,icarbon) + &
            LC(j,ilabile)     * turnover(:,j,ilabile,icarbon)


       lignin_struc_inc(:,j,ibelow) = &
            LC(j,isapbelow)   * bm_to_litter(:,j,isapbelow,icarbon) + &
            LC(j,iheartbelow) * bm_to_litter(:,j,iheartbelow,icarbon) + &
            LC(j,iroot)       * bm_to_litter(:,j,iroot,icarbon) + &
            LC(j,isapbelow)   * turnover(:,j,isapbelow,icarbon) + &
            LC(j,iheartbelow) * turnover(:,j,iheartbelow,icarbon) + &
            LC(j,iroot)       * turnover(:,j,iroot,icarbon)

       !! 2.2.4 lignin increase in woody litter
       
       lignin_wood_inc(:,j,iabove) = &
            LC(j,isapabove)   * bm_to_litter(:,j,isapabove,icarbon) + &
            LC(j,iheartabove) * bm_to_litter(:,j,iheartabove,icarbon) + &
            LC(j,isapabove)   * turnover(:,j,isapabove,icarbon) + &
            LC(j,iheartabove) * turnover(:,j,iheartabove,icarbon) 
       
       lignin_wood_inc(:,j,ibelow) = &
            LC(j,isapbelow)   * bm_to_litter(:,j,isapbelow,icarbon) + &
            LC(j,iheartbelow) * bm_to_litter(:,j,iheartbelow,icarbon) + &
            LC(j,isapbelow)   * turnover(:,j,isapbelow,icarbon) + &
            LC(j,iheartbelow) * turnover(:,j,iheartbelow,icarbon) 
       
    ENDDO
	
	frac_metabave(:,:,:)= zero
	WHERE (litter_inc(:,imetabolic,:,:,icarbon).GT.zero)
		frac_metabave(:,:,:)=litter_inc(:,imetabolic,:,:,icarbon)/(litter_inc(:,imetabolic,:,:,icarbon)+ &
			litter_inc(:,istructural,:,:,icarbon)+litter_inc(:,iwoody,:,:,icarbon))
	ENDWHERE
	
	! Determinging whether do bare fallow experiment, HZ
	npp_longtermave(:,:)=npp_longterm(:,:)
	IF(isbarefallow) THEN
        litter_inc(:,:,:,:,:)=zero 
		npp_longtermave(:,:)=zero 
	ENDIF
    !! 2.2.5 add new litter (struct/met, above/below)
!	IF(dsg_debug) THEN 
	!	WRITE (numout,*) 'ZHC1_stomate_litter: mass check'
	!	DO k=1,3
	!		DO i=1,1
	!			DO m=7,7
	!				WRITE (numout,*) ' element :', k, 'grid', i, 'pft',m
	!				WRITE (numout,*) ' Litter_input_ab', sum(litter_inc(i,:,m,iabove,k))
	!				WRITE (numout,*) ' Litter_input_be', sum(litter_inc(i,:,m,ibelow,k))
	!				WRITE (numout,*) ' Litter_input', sum(turnover(i,m,:,k))+sum(bm_to_litter(i,m,:,k))
	!				WRITE (numout,*) ' Microbes_ab', microbe(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Microbes_be', microbe(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Litter_ab', litter(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Litter_be', litter(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Repiration', resp_hetero_litter(i,m)
	!				WRITE (numout,*) ' som_input',som_input(i,:,m,k)
	!				WRITE (numout,*) ' Nmin', n_mineralisation(i,m)
	!				WRITE (numout,*) ' Pmin', p_mineralisation(i,m)
	!			ENDDO
	!		ENDDO
	!	ENDDO
	
!		WRITE (numout,*) ' litt2som', frac_litsom(1,:,1)
!		WRITE (numout,*) ' litt2som', frac_litsom(2,:,1)
!		WRITE (numout,*) ' litt2som', frac_litsom(3,:,1)
!		WRITE (numout,*) ' litt2som', frac_litsom(1,:,2)
!		WRITE (numout,*) ' litt2som', frac_litsom(2,:,2)
!		WRITE (numout,*) ' litt2som', frac_litsom(3,:,2)
!		WRITE (numout,*) ' un', un
!	ENDIF
	
	DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO k = 1,nlitt !Lopp litter types
		  DO j = 1, ncarb
			 som_input(:,j,:,icarbon)=som_input(:,j,:,icarbon) + litter_inc(:,k,:,l,icarbon)*frac_litsom(k,j,l)
			 n_mineralisation(:,:)= n_mineralisation(:,:)+litter_inc(:,k,:,l,initrogen)*frac_litsom(k,j,l)
			 p_mineralisation(:,:)= p_mineralisation(:,:)+litter_inc(:,k,:,l,iphosphorus)*frac_litsom(k,j,l)
!			 WRITE (numout,*) ' layer', l, 'litt', k, 'icarb',j,'sominput',som_input(2,j,3,icarbon), &
!								'litter_inc',litter_inc(2,k,3,l,icarbon),'fraclitsom',frac_litsom(k,j,l)
		  ENDDO
		  litter_inc(:,k,:,l,:)=litter_inc(:,k,:,l,:)*(un - frac_litsom(k,isomp,l)-frac_litsom(k,isomc,l)-frac_litsom(k,isoma,l))
	   ENDDO
	   lignin_struc_inc(:,:,l)=lignin_struc_inc(:,:,l) * (un -frac_litsom(istructural,isomc,l))
	   lignin_wood_inc(:,:,l)=lignin_wood_inc(:,:,l) * (un -frac_litsom(iwoody,isomc,l))
	ENDDO
    litter(:,:,:,:,:) = litter(:,:,:,:,:) + litter_inc(:,:,:,:,:)
	!
	!IF(dsg_debug) THEN 
	!	WRITE (numout,*) 'ZHC2_stomate_litter: mass check'
	!	DO k=1,3
	!		DO i=1,1
	!			DO m=7,7
	!				WRITE (numout,*) ' element :', k, 'grid', i, 'pft',m
	!				WRITE (numout,*) ' Litter_input_ab', sum(litter_inc(i,:,m,iabove,k))
	!				WRITE (numout,*) ' Litter_input_be', sum(litter_inc(i,:,m,ibelow,k))
	!				WRITE (numout,*) ' Microbes_ab', microbe(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Microbes_be', microbe(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Litter_ab', litter(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Litter_be', litter(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Repiration', resp_hetero_litter(i,m)
	!				WRITE (numout,*) ' som_input',som_input(i,:,m,k)
	!				WRITE (numout,*) ' Nmin', n_mineralisation(i,m)
	!				WRITE (numout,*) ' Pmin', p_mineralisation(i,m)
	!			ENDDO
	!		ENDDO
	!	ENDDO
	!
    !gmjc for grazing litter
    IF (do_slow) THEN
      DO j=2,nvm
        !! grazed grassland or natural grassland with wild animal
        IF ((is_grassland_manag(j) .AND. is_grassland_grazed(j)) .OR. &
           ((.NOT. is_tree(j)) .AND. natural(j) .AND. &
             & (.NOT. is_grassland_cut(j)) .AND. (.NOT.is_grassland_grazed(j)))) THEN
          WHERE (litter(:,:,j,iabove,icarbon) .GE. litter_not_avail(:,:,j) &
             .AND. litter(:,:,j,iabove,icarbon) .GT. zero)
            litter_avail_frac(:,:,j) = (litter(:,:,j,iabove,icarbon) - litter_not_avail(:,:,j)) &
                  & / litter(:,:,j,iabove,icarbon)
          ! if litter not available equal to or larger than litter
          ELSEWHERE
            litter_avail_frac(:,:,j) = zero
          ENDWHERE
          IF (ANY(litter_avail_frac .LT. zero)) THEN
            WRITE (numout,*) 'frac error',litter(:,:,j,iabove,icarbon),litter_not_avail(:,:,j),litter_avail_frac(:,:,j)
            STOP 'error fraction litter available for grazing < 0'
          ENDIF
        ELSE
          litter_avail_frac(:,:,j) = un
        ENDIF
      ENDDO
    ENDIF
    !end gmjc for grazing litter

    !! 2.2.6 for security: can't add more lignin than structural litter (above/below)
    DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO m = 1,nvm !Lopp over PFTs

          lignin_struc_inc(:,m,l) = &
               MIN( lignin_struc_inc(:,m,l), litter_inc(:,istructural,m,l,icarbon) )
          lignin_wood_inc(:,m,l) = &
               MIN( lignin_wood_inc(:,m,l), litter_inc(:,iwoody,m,l,icarbon) )

       ENDDO
    ENDDO


    !! 2.2.7 new lignin content: add old lignin and lignin increase, divide by 
    !!       total structural litter (above/below)

    WHERE ( litter(:,istructural,:,:,icarbon) .GT. min_stomate )
       lignin_struc(:,:,:) =  &
            ( lignin_struc(:,:,:)*old_struc(:,:,:) + lignin_struc_inc(:,:,:) ) / &
            litter(:,istructural,:,:,icarbon)
    ELSEWHERE
       lignin_struc(:,:,:) = zero
    ENDWHERE

    WHERE ( litter(:,iwoody,:,:,icarbon) .GT. min_stomate )
       lignin_wood(:,:,:) =  &
            ( lignin_wood(:,:,:)*old_woody(:,:,:) + lignin_wood_inc(:,:,:) ) / &
            litter(:,iwoody,:,:,icarbon)
    ELSEWHERE
       lignin_wood(:,:,:) = zero
    ENDWHERE
    

    !! 3. Temperature control on decay: Factor between 0 and 1
	control_temp(:,:)=zero
	control_tempvmax(:,:)=zero
	control_tempkm(:,:,:)=zero
	control_temptau(:,:)=zero
	control_tempmictau(:,:)=zero
	control_tempdesorp(:,:)=zero						 
    !! 3.1 above: surface temperature
    control_temp(:,iabove) = control_temp_func (npts, tsurf)
	control_tempvmax(:,iabove) = control_tempvmax_func (npts, tsurf)
	control_tempkm(:,:,iabove) = control_tempkm_func (npts, tsurf) ! control of tempreture on metabolic and SOMa
	control_temptau(:,iabove) = control_temptau_func (npts, tsurf, tsurf_longterm) ! control of tempreture on metabolic and SOMa
	control_tempdesorp(:,iabove)= control_tempdesorp_func (npts, tsurf_longterm) ! control of tempreture on SOC desorption
	control_tempmictau(:,iabove) = control_tempmictau_func (npts, tsurf, tsurf_longterm) ! control of tempreture on metabolic and SOMa
    
    !! 3.2 below: convolution of temperature and decomposer profiles
    !!            (exponential decomposer profile supposed)
   
    !! 3.2.1 rpc is an integration constant such that the integral of the root profile is 1.
    rpc(:) = un / ( un - EXP( -z_soil(nslm) / z_decomp ) )

    !! 3.2.2 integrate over the nslm levels
    tsoil_decomp(:) = zero
	tsoil_decomplong(:) = zero					   

    DO l = 1, nslm

       tsoil_decomp(:) = &
            tsoil_decomp(:) + tsoil(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )
	   tsoil_decomplong(:) = &
            tsoil_decomplong(:) + tsoil_longterm(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )

    ENDDO

    control_temp(:,ibelow) = control_temp_func (npts, tsoil_decomplong)
	control_tempvmax(:,ibelow) = control_tempvmax_func (npts, tsoil_decomp)
	control_tempkm(:,:,ibelow) = control_tempkm_func (npts, tsoil_decomp)
	control_temptau(:,ibelow) = control_temptau_func (npts, tsoil_decomp,tsoil_decomplong)
    control_tempdesorp(:,ibelow)= control_tempdesorp_func (npts, tsoil_decomplong) ! control of tempreture on SOC desorption
	control_tempmictau(:,ibelow) = control_tempmictau_func (npts, tsoil_decomp,tsoil_decomplong)
	
  !! 4. Moisture control. Factor between 0 and 1
    !! Current model does not consider the impacts of moisture on micrbial decomposition rate
    !! 4.1 above the ground: litter humidity
    control_moist(:,iabove) = control_moist_func (npts, litterhum_longterm)
	control_moistvmax(:,iabove) = control_moistvmax_func (npts, litterhum, bulk, clay)																			   
	control_moistau(:,iabove) = control_moistau_func (npts, litterhum, litterhum_longterm)

    !
    !! 4.2 below: convolution of humidity and decomposer profiles
    !            (exponential decomposer profile supposed)

    !! 4.2.1 rpc is an integration constant such that the integral of the root profile is 1.
    rpc(:) = un / ( un - EXP( -z_soil(nslm) / z_decomp ) )

    !! 4.2.2 integrate over the nslm levels
    soilhum_decomp(:) = zero
	soilhumlong_decomp(:) = zero
    DO l = 1, nslm !Loop over soil levels

       soilhum_decomp(:) = &
            soilhum_decomp(:) + soilhum(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )
	   soilhumlong_decomp(:) = &
            soilhumlong_decomp(:) + soilhum_longterm(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )

    ENDDO

    control_moist(:,ibelow) = control_moist_func (npts, soilhumlong_decomp)
	control_moistvmax(:,ibelow) = control_moistvmax_func (npts, soilhum_decomp, bulk, clay)																					
	control_moistau(:,ibelow) = control_moistau_func (npts, soilhum_decomp, soilhumlong_decomp)
	
	! Combined the controlling factor of temperature and soil moisture into one
	control_tempvmax(:,:) = control_tempvmax(:,:)*control_moistvmax(:,:)																 
	!WRITE (numout,*) 'ZHC1_stomatelitter'
	!WRITE (numout,*) 'tem_surf',tsurf(1),'tem_surf_longterm',tsurf_longterm(1)
	!WRITE (numout,*) 'tem_soil',tsoil_decomp(1),'tem_soil_longterm',tsoil_decomplong(1)
	!WRITE (numout,*) 'control_tempvmax',control_tempvmax(1,:)
	!WRITE (numout,*) 'control_tempkm',control_tempkm(1,1,:)
	!WRITE (numout,*) 'control_temptau',control_temptau(1,:)
	!WRITE (numout,*) 'soilhum_decomp',soilhum_decomp, 'soilhumlong_decomp',soilhumlong_decomp
	!WRITE (numout,*) 'litterhum',litterhum, 'litterhum_longterm',litterhum_longterm
	!WRITE (numout,*) 'control_moistau',control_moistau(1,:)
  !! 5. Litter decomposition and the Turnover of microbial biomass (mg C cm^-3)	
  ! The decomposition of LIT and SOMa pools follows Michaelis–Menten kinetics (Wieder et al., 2015)
  !
  !! 5.1 Concentration of litter and microbial biomass (mg C cm^-3)
  ! HZ: Different from CENTURY, MIMICS calulate the decompositon rate of organic matters based on the
  ! concentration (mgC cm-3) rather than stock (gC m-2), Thus, it is necessary to calculate:
  ! 1) total litter-C stock (gC cm-2) and total litter (not only the quantity of litter-C) stock
  ! 2) total volume of the surface litter layer (based on a empirical equation obtained from ***)
  ! 3) Concentration of each litter pool (metabolic, structural, woody)
  !
	mass_above(:,:)=zero
	depth_litter(:,:)=zero
	bd_litter(:,:)=zero
	
!	WRITE(numout,*) 'ZHC02_stomate_micr_ab:',microbe(1,imicr,:,iabove,icarbon)
!	WRITE(numout,*) 'ZHC02_stomate_mick_ab:',microbe(1,imick,:,iabove,icarbon)
!	WRITE(numout,*) 'ZHC02_stomate_micr_be:',microbe(1,imicr,:,ibelow,icarbon)
!	WRITE(numout,*) 'ZHC02_stomate_mick_be:',microbe(1,imick,:,ibelow,icarbon)
	!npts,nlitt,nvm,nlevs,nelements  npts,nlitt,nvm,nlevs
	mass_above(:,:) = 1.0E-4*(litter(:,imetabolic,:,iabove,icarbon)+litter(:,istructural,:,iabove,icarbon)+ &
					  litter(:,iwoody,:,iabove,icarbon)+microbe(:,imicr,:,iabove,icarbon) + &
					  microbe(:,imick,:,iabove,icarbon))/c2om_ratio 				! g cm^2
	WHERE (mass_above(:,:).GT.zero)
		bd_litter(:,:) = coef_littbd1*((mass_above(:,:))**coef_littbd2)             ! g cm^-3
	ENDWHERE
	!
	WHERE (bd_litter(:,:).GT.zero)
		depth_litter(:,:) = mass_above(:,:)/bd_litter(:,:)              			! cm
	ENDWHERE
	!
	!IF(dsg_debug) THEN 
	!	WRITE (numout,*) 'ZHC1_stomatelitter: mass check'
	!	DO i=1,npts
	!		DO m=1,nvm
	!			WRITE (numout,*) 'igrid',i , ' PFT: ',m ,  &
	!							 'litter_depth', depth_litter(i,m), &
	!							 'mass_above',mass_above(i,m), &
	!							 'coef1,2',coef_littbd1,coef_littbd2, &
	!							 'density_litter',bd_litter(i,m), &
	!							 'litter_stock',litter(i,:,m,iabove,icarbon), &
	!							 'microbe_stock',microbe(i,:,m,iabove,icarbon)
	!		ENDDO
	!	ENDDO
	!ENDIF
	!
	litter_density(:,:,:,:,:)=zero
	som_density(:,:,:,:)=zero
	microbe_density(:,:,:,:,:)=zero
	microbe_active(:,:,:,:,:)=zero
	microbe_dormant(:,:,:,:,:)=zero
	frac_micactive(:,:,:,:)=zero
	!
	DO  k = 1, nlitt
		DO j = 1,nelements
		   IF (k==iwoody) THEN
			  WHERE (depth_litter(:,:).GT.zero)
				litter_density(:,k,:,iabove,j) = 0.1*litter(:,k,:,iabove,j)/depth_litter(:,:) * frac_woody_avail 	! mg C cm^-3
			  ENDWHERE
			  litter_density(:,k,:,ibelow,j) = 0.1*litter(:,k,:,ibelow,j)/(100.0*z_mineralsoil)* frac_woody_avail  	! mg C cm^-3
		   ELSE
			  WHERE (depth_litter(:,:).GT.zero)
				litter_density(:,k,:,iabove,j) = 0.1*litter(:,k,:,iabove,j)/depth_litter(:,:) 		! mg C cm^-3
			  ENDWHERE
		      litter_density(:,k,:,ibelow,j) = 0.1*litter(:,k,:,ibelow,j)/(100.0*z_mineralsoil)  	! mg C cm^-3		   
		   ENDIF
		ENDDO
	ENDDO
	!
	som_density(:,:,:,:) = 0.1*som(:,:,:,:)/(100.0*z_mineralsoil)  	! mg C cm^-3
	!
	DO  k = 1, nmic
		DO j = 1,nelements
		   WHERE (depth_litter(:,:).GT.zero)
		     microbe_density(:,k,:,iabove,j) = 0.1*microbe(:,k,:,iabove,j)/depth_litter(:,:)  		! mg C cm^-3
		   ENDWHERE
		   microbe_density(:,k,:,ibelow,j) = 0.1*microbe(:,k,:,ibelow,j)/(100.0*z_mineralsoil)  	! mg C cm^-3
		ENDDO
	ENDDO
   !
   DO l = 1, nlevs
	   IF (l == iabove) THEN
		!	frac_micactive(:,imicr,:,l)=MAX(min_stomate, un-EXP(-coef_fmicract*litter_density(:,imetabolic,:,l,icarbon)))
		!	frac_micactive(:,imick,:,l)=MAX(min_stomate, un-EXP(-coef_fmickact*(litter_density(:,istructural,:,l,icarbon)+ &
		!							litter_density(:,iwoody,:,l,icarbon))))
		frac_micactive(:,imicr,:,l)=un
		frac_micactive(:,imick,:,l)=un
	   ELSE
		!	frac_micactive(:,imicr,:,l)=MAX(min_stomate, un-EXP(-coef_fmicract*(litter_density(:,imetabolic,:,l,icarbon)+ &
		!							som_density(:,isoma,:,icarbon))))
		!	frac_micactive(:,imick,:,l)=MAX(min_stomate, un-EXP(-coef_fmickact*(litter_density(:,istructural,:,l,icarbon)+ &
		!							litter_density(:,iwoody,:,l,icarbon)+som_density(:,isoma,:,icarbon))))
		frac_micactive(:,imicr,:,l)=un
		frac_micactive(:,imick,:,l)=un
	   ENDIF
   ENDDO
   
   DO  k = 1, nmic
     DO l = 1, nlevs
       DO m = 1, nvm
	       frac_micactive(:,k,m,l)=MIN(frac_micactmax,frac_micactive(:,k,m,l)*MIN(control_temptau(:,l),control_moistau(:,l)))
	   ENDDO
	 ENDDO
   ENDDO
   ! Output with XIOS
   CALL xios_orchidee_send_field("factive_MICr_ab",frac_micactive(:,imicr,:,iabove))
   CALL xios_orchidee_send_field("factive_MICk_ab",frac_micactive(:,imick,:,iabove))
   CALL xios_orchidee_send_field("factive_MICr_be",frac_micactive(:,imicr,:,ibelow))
   CALL xios_orchidee_send_field("factive_MICk_be",frac_micactive(:,imick,:,ibelow))
   ! Output with IOIPSL
   CALL histwrite_p (hist_id_stomate, 'factive_MICr_ab', itime, & 
            frac_micactive(:,imicr,:,iabove), npts*nvm, horipft_index)
   CALL histwrite_p (hist_id_stomate, 'factive_MICk_ab', itime, & 
            frac_micactive(:,imick,:,ibelow), npts*nvm, horipft_index)
   CALL histwrite_p (hist_id_stomate, 'factive_MICr_be', itime, & 
            frac_micactive(:,imicr,:,iabove), npts*nvm, horipft_index)
   CALL histwrite_p (hist_id_stomate, 'factive_MICk_be', itime, & 
            frac_micactive(:,imick,:,ibelow), npts*nvm, horipft_index)
   !
   DO  k = 1, nmic
    DO l = 1, nlevs
     DO m = 1, nvm !Loop over PFTs
	  DO j = 1, nelements
		 microbe_active(:,k,m,l,j) = microbe_density(:,k,m,l,j)*frac_micactive(:,k,m,l)
		 !* & MIN(control_temptau(:,l),control_moistau(:,l));
		 microbe_dormant(:,k,m,l,j) = microbe_density(:,k,m,l,j)-microbe_active(:,k,m,l,j);
      ENDDO
     ENDDO
    ENDDO
   ENDDO
   
   !WRITE (numout,*) 'ZHC2_stomatelitter'
   !WRITE (numout,*) 'frac_micactive_r',frac_micactive(1,imicr,7,:)
   !WRITE (numout,*) 'frac_micactive_k',frac_micactive(1,imick,7,:)
   !WRITE (numout,*) 'microbe_density_r',microbe_density(1,imicr,7,:,icarbon)
   !WRITE (numout,*) 'microbe_density_k',microbe_density(1,imick,7,:,icarbon)
   !WRITE (numout,*) 'microbe_active_r',microbe_active(1,imicr,7,:,icarbon)
   !WRITE (numout,*) 'microbe_active_k',microbe_active(1,imick,7,:,icarbon)
   !WRITE (numout,*) 'microbe_dormant_r',microbe_dormant(1,imicr,7,:,icarbon)
   !WRITE (numout,*) 'microbe_dormant_k',microbe_dormant(1,imick,7,:,icarbon)
   !WRITE (numout,*) 'microbe_r',microbe(1,imicr,7,:,icarbon)
   !WRITE (numout,*) 'microbe_k',microbe(1,imick,7,:,icarbon)
   !WRITE (numout,*) 'litterimetabolic',litter(1,imetabolic,7,:,icarbon)
   !WRITE (numout,*) 'litteristructural',litter(1,istructural,7,:,icarbon)
   !WRITE (numout,*) 'litteriwoody',litter(1,iwoody,7,:,icarbon)
   !WRITE (numout,*) 'LITmet_CN_CP',litter(1,imetabolic,7,:,icarbon)/litter(1,imetabolic,7,:,initrogen), &
	!								litter(1,imetabolic,7,:,icarbon)/litter(1,imetabolic,7,:,iphosphorus)
   !WRITE (numout,*) 'LITstr_CN_CP',litter(1,istructural,7,:,icarbon)/litter(1,istructural,7,:,initrogen), &
	!								litter(1,istructural,7,:,icarbon)/litter(1,istructural,7,:,iphosphorus)
   !WRITE (numout,*) 'LITwod_CN_CP',litter(1,iwoody,7,:,icarbon)/litter(1,iwoody,7,:,initrogen), &
	!								litter(1,iwoody,7,:,icarbon)/litter(1,iwoody,7,:,iphosphorus)
   !WRITE (numout,*) 'litter_densityimetabolic',litter(1,imetabolic,7,:,icarbon)
   !WRITE (numout,*) 'litter_densityistructural',litter(1,istructural,7,:,icarbon)
   !WRITE (numout,*) 'litter_densityiwoody',litter(1,iwoody,7,:,icarbon)
   !IF(dsg_debug) THEN 
	!	WRITE (numout,*) 'ZHC2_stomatelitter: mass check'
	!	DO i=1,1
	!		DO m=7,7
	!			WRITE (numout,*) 'igrid',i , ' PFT: ', m
	!			WRITE (numout,*) 'litter_density',litter_density(1,:,m,iabove,icarbon)
	!			WRITE (numout,*) 'Density_microbe',microbe_density(1,:,m,1,1)
	!			WRITE (numout,*) 'frac_micactive',frac_micactive(1,:,m,iabove),frac_micactive(1,:,m,ibelow)
	!		ENDDO
	!	ENDDO
	!ENDIF
   !
   !! 5.2 Maximum microbial reaction velocity and the half saturation of substrate concentrations	
	vmax_mic(:,:,:)=zero
	khalf_mic(:,:,:)=zero
	
	DO l = 1, nlevs
	   IF (l == iabove) THEN
		  vmax_mic(:,:,l)=vmax_modlitt(:,:,l)*(av_vmax_mod*av_vmax_mic)*av_vmax_soil
		  WHERE (khalf_modlitt(:,:,l).GT.zero)
			khalf_mic(:,:,l)=((ak_khalf_mod*ak_khalf_mic)/khalf_modlitt(:,:,l))*ak_khalf_soil
		  ENDWHERE
	   ELSE
		  vmax_mic(:,:,l)=vmax_modlitt(:,:,l)*(av_vmax_mod*av_vmax_mic)
		  WHERE (khalf_modlitt(:,:,l).GT.zero)
			khalf_mic(:,:,l)=(ak_khalf_mod*ak_khalf_mic)/khalf_modlitt(:,:,l)
		  ENDWHERE
	   ENDIF
	ENDDO
  !! 5.3 Carbon Use Efficiency of microbes when consuming different 
  !      litter substrates (metabolic, structural, woody)
  
	frac_litt_cue(:,:,:,:,:)=zero
	
	! Fixed microbial carbon use efficiency
	DO m = 1, nvm !Loop over PFTs
		DO k = 1, nlitt
			frac_litt_cue(:,m,k,imicr,iabove)=frac_soil(k,imicr,iabove) ! Fraction of decayed aboveground litter to MIC-r
			frac_litt_cue(:,m,k,imick,iabove)=frac_soil(k,imick,iabove) ! Fraction of decayed aboveground litter to MIC-k
			frac_litt_cue(:,m,k,imicr,ibelow)=frac_soil(k,imicr,ibelow) ! Fraction of decayed belowground litter to MIC-r
			frac_litt_cue(:,m,k,imick,ibelow)=frac_soil(k,imick,ibelow) ! Fraction of decayed belowground litter to MIC-k
		ENDDO ! k = 1, nlitt
	ENDDO !m = 1,nvm
	
	!WRITE (numout,*) 'ZHC2_stomatelitter'
    !WRITE (numout,*) 'frac_litt_cue:imetabolic,imicr',frac_litt_cue(1,7,imetabolic,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:istructural,imicr',frac_litt_cue(1,7,istructural,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:iwoody,imicr',frac_litt_cue(1,7,iwoody,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:imetabolic,imicr',frac_litt_cue(1,7,imetabolic,imick,:)
	!WRITE (numout,*) 'frac_litt_cue:istructural,imicr',frac_litt_cue(1,7,istructural,imick,:)
	!WRITE (numout,*) 'frac_litt_cue:iwoody,imicr',frac_litt_cue(1,7,iwoody,imick,:)
	!WRITE (numout,*) 'coef',cuemax_micr, coef_cuen, coef_cuenconr,coef_cuep,coef_cuepconr
	!WRITE (numout,*) 'mineralNP',soil_n_min(1,7,iammonium),soil_n_min(1,7,initrate),soil_p_min(1,7,ipdissolved)
	!WRITE (numout,*) 'micCNP',CN_microbe(1,7,:),CP_microbe(1,7,:)
	
	IF(miccue_dynamic) THEN 
		! Dynamic microbial carbon use efficiency
	  DO m = 1, nvm !Loop over PFTs
		DO k = 1, nlitt
			! Fraction of decayed aboveground litter to MIC-r
			WHERE (litter(:,k,m,iabove,initrogen).GT.zero .AND. litter(:,k,m,iabove,iphosphorus).GT.zero)
				frac_litt_cue(:,m,k,imicr,iabove)= &
					MIN(cuemax_micr,cuemax_micr*(MAX(un,litter(:,k,m,iabove,icarbon)/litter(:,k,m,iabove,initrogen)/CN_mictarget(:,m,imicr)))** &
					(coef_cuen*(MIN(zero,(soil_n_min(:,m,iammonium)+soil_n_min(:,m,initrate))/(0.3*bulk(:))-coef_cuenconr))), &
					cuemax_micr*(MAX(un,litter(:,k,m,iabove,icarbon)/litter(:,k,m,iabove,iphosphorus)/CP_mictarget(:,m,imicr)))** &
					(coef_cuep*(MIN(zero,soil_p_min(:,m,ipdissolved)/(0.3*bulk(:))-coef_cuepconr))))
			ELSEWHERE
				frac_litt_cue(:,m,k,imicr,iabove)=frac_soil(k,imicr,iabove)
			ENDWHERE
			! Fraction of decayed aboveground litter to MIC-k
			WHERE (litter(:,k,m,iabove,initrogen).GT.zero .AND. litter(:,k,m,iabove,iphosphorus).GT.zero)
				frac_litt_cue(:,m,k,imick,iabove)= &
					MIN(cuemax_mick,cuemax_mick*(MAX(un,litter(:,k,m,iabove,icarbon)/litter(:,k,m,iabove,initrogen)/CN_mictarget(:,m,imick)))** &
					(coef_cuen*(MIN(zero,(soil_n_min(:,m,iammonium)+soil_n_min(:,m,initrate))/(0.3*bulk(:))-coef_cuenconk))), &
					cuemax_mick*(MAX(un,litter(:,k,m,iabove,icarbon)/litter(:,k,m,iabove,iphosphorus)/CP_mictarget(:,m,imick)))** &
					(coef_cuep*(MIN(zero,soil_p_min(:,m,ipdissolved)/(0.3*bulk(:))-coef_cuepconk))))
			ELSEWHERE
				frac_litt_cue(:,m,k,imick,iabove)=frac_soil(k,imick,iabove)
			ENDWHERE
			! Fraction of decayed belowground litter to MIC-r
			WHERE (litter(:,k,m,ibelow,initrogen).GT.zero .AND. litter(:,k,m,ibelow,iphosphorus).GT.zero)
				frac_litt_cue(:,m,k,imicr,ibelow)= &
					MIN(cuemax_micr,cuemax_micr*(MAX(un,litter(:,k,m,ibelow,icarbon)/litter(:,k,m,ibelow,initrogen)/CN_mictarget(:,m,imicr)))** &
					(coef_cuen*(MIN(zero,(soil_n_min(:,m,iammonium)+soil_n_min(:,m,initrate))/(0.3*bulk(:))-coef_cuenconr))), &
					cuemax_micr*(MAX(un,litter(:,k,m,ibelow,icarbon)/litter(:,k,m,ibelow,iphosphorus)/CP_mictarget(:,m,imicr)))** &
					(coef_cuen*(MIN(zero,soil_p_min(:,m,ipdissolved)/(0.3*bulk(:))-coef_cuepconr))))
			ELSEWHERE
				frac_litt_cue(:,m,k,imicr,ibelow)=frac_soil(k,imicr,ibelow)
			ENDWHERE
			! Fraction of decayed belowground litter to MIC-k
			WHERE (litter(:,k,m,ibelow,initrogen).GT.zero .AND. litter(:,k,m,ibelow,iphosphorus).GT.zero)
				frac_litt_cue(:,m,k,imick,ibelow)= &
					MIN(cuemax_mick,cuemax_mick*(MAX(un,litter(:,k,m,ibelow,icarbon)/litter(:,k,m,ibelow,initrogen)/CN_mictarget(:,m,imick)))** &
					(coef_cuen*(MIN(zero,(soil_n_min(:,m,iammonium)+soil_n_min(:,m,initrate))/(0.3*bulk(:))-coef_cuenconk))), &
					cuemax_mick*(MAX(un,litter(:,k,m,ibelow,icarbon)/litter(:,k,m,ibelow,iphosphorus)/CP_mictarget(:,m,imick)))** &
					(coef_cuen*(MIN(zero,soil_p_min(:,m,ipdissolved)/(0.3*bulk(:))-coef_cuepconk))))
			ELSEWHERE
				frac_litt_cue(:,m,k,imick,ibelow)=frac_soil(k,imick,ibelow)
			ENDWHERE
		
		ENDDO ! k = 1, nlitt
	  ENDDO !m = 1,nvm
	ENDIF !IF(miccue_dynamic) THEN 
	
	!WRITE (numout,*) 'ZHC3_stomatelitter'
    !WRITE (numout,*) 'frac_litt_cue:imetabolic,imicr',frac_litt_cue(1,7,imetabolic,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:istructural,imicr',frac_litt_cue(1,7,istructural,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:iwoody,imicr',frac_litt_cue(1,7,iwoody,imicr,:)
	!WRITE (numout,*) 'frac_litt_cue:imetabolic,imicr',frac_litt_cue(1,7,imetabolic,imick,:)
	!WRITE (numout,*) 'frac_litt_cue:istructural,imicr',frac_litt_cue(1,7,istructural,imick,:)
	!WRITE (numout,*) 'frac_litt_cue:iwoody,imicr',frac_litt_cue(1,7,iwoody,imick,:)
  !	
  !! 5.4 Decomposition of litter substrates (metabolic, structural, woody)
  !      & Turnover of microbes
  !
  	
	frac_micsom(:,:,:,:,:) = zero !npts,nvm,nmic,ncarb,nlevs
	tau_mic(:,:,:,:)=zero
	cflux_litt2mic(:,:,:,:)=zero
	cflux_litt(:,:,:,:)=zero
	cflux_mic(:,:,:,:)=zero
	fdecomp_np=un
	np_supply=zero	
    np_need=zero
    soilN_density(:,:)=	(soil_n_min(:,:,iammonium)+soil_n_min(:,:,initrate))/(100.0*z_mineralsoil*10.0) ! mg N cm-3
	soilP_density(:,:)=	soil_n_min(:,:,ipdissolved)/(100.0*z_mineralsoil*10.0) ! mg P cm-3
	!
	DO m = 1,nvm !Loop over PFTs
      DO l = 1, nlevs !Loop over litter levels (above and below ground)
		DO k = 1, nlitt !Loop over litter types
		  !5.4.1 Decomposed litter-C (mg C cm^-3)
		  qd(:,:)=zero !Unit: mg C cm^-3
		  qd_mic(:,:,:)=zero !Unit: mg C cm^-3
		  vmax_lignin(:)=zero ! Unitless (0-1)
		  IF (k.EQ.imetabolic) THEN
			  vmax_lignin(:) = un
		  ELSEIF (k.EQ.istructural) THEN
		      vmax_lignin(:) = un
			  !vmax_lignin(:) = exp( -litter_struct_coef * lignin_struc(:,m,l))
		  ELSEIF (k.EQ.iwoody) THEN
			  vmax_lignin(:) = EXP( -litter_woody_coef * lignin_struc(:,m,l))
		  ENDIF
		  DO i=1,npts
			IF (litter_density(i,k,m,l,icarbon).GT.zero) THEN
				qd_mic(i,imicr,icarbon)=litter_density(i,k,m,l,icarbon)*microbe_active(i,imicr,m,l,icarbon)* &
					control_tempvmax(i,l)*vmax_mic(k,imicr,l)*vmax_lignin(i)/ &
					(control_tempkm(i,k,l)*khalf_mic(k,imicr,l) + litter_density(i,k,m,l,icarbon))
				qd_mic(i,imick,icarbon)=litter_density(i,k,m,l,icarbon)*microbe_active(i,imick,m,l,icarbon)* &
					control_tempvmax(i,l)*vmax_mic(k,imick,l)*vmax_lignin(i)/ &
					(control_tempkm(i,k,l)*khalf_mic(k,imick,l) + litter_density(i,k,m,l,icarbon))
				
				!IF(m.EQ.7 .AND. i.EQ.1) THEN
				!	WRITE (numout,*) 'ZHCcue1_stomatelitter'
				!	WRITE (numout,*) 'ipft',m,'isoilly',l,'ilitt',k
				!	WRITE (numout,*) 'litter_density',litter_density(1,k,m,l,icarbon),'microbe_active',microbe_active(1,:,m,l,icarbon)
				!	WRITE (numout,*) 'control_tempvmax',control_tempvmax(1,l),'vmax_mic',vmax_mic(k,:,l),'vmax_lignin',vmax_lignin(1)
				!	WRITE (numout,*) 'control_tempkm',control_tempkm(1,k,l),'khalf_mic',khalf_mic(k,:,l)
				!	WRITE (numout,*) 'qd',qd(1,icarbon),'qd_mic',qd_mic(1,:,icarbon)
				!	WRITE (numout,*) 'cflux_litt2mic',cflux_litt2mic(1,:,m,l)
				!	WRITE (numout,*) 'frac_litt_cue', frac_litt_cue(i,m,k,imicr,l),frac_litt_cue(i,m,k,imick,l)
				!ENDIF
				
			!	(1) MICr - N limit
				np_supply= qd_mic(i,imicr,icarbon)*litter_density(i,k,m,l,initrogen)/litter_density(i,k,m,l,icarbon)+ &
							MAX(frac_soil_np*(soilN_density(i,m)-min_soil_mineralN),zero)
				np_need = qd_mic(i,imicr,icarbon)/CN_mictarget(i,m,imicr)
				IF(np_need.GT.zero) THEN
					frac_litt_cue(i,m,k,imicr,l)=MAX(cuemin_som, MIN(frac_litt_cue(i,m,k,imicr,l),np_supply/np_need))
				ENDIF
			!	(2) MICr - P limit
				np_supply= qd_mic(i,imicr,icarbon)*litter_density(i,k,m,l,iphosphorus)/litter_density(i,k,m,l,icarbon)+ &
							MAX(frac_soil_np*(soilP_density(i,m)-min_soil_mineralP),zero)
				np_need = qd_mic(i,imicr,icarbon)/CP_mictarget(i,m,imicr)
				IF(np_need.GT.zero) THEN
					frac_litt_cue(i,m,k,imicr,l)=MAX(cuemin_som, MIN(frac_litt_cue(i,m,k,imicr,l),np_supply/np_need))
				ENDIF				
			!	(3) MICk - N limit
				np_supply= qd_mic(i,imick,icarbon)*litter_density(i,k,m,l,initrogen)/litter_density(i,k,m,l,icarbon)+ &
						   MAX(frac_soil_np*(soilN_density(i,m)-min_soil_mineralN),zero)
				np_need = qd_mic(i,imick,icarbon)/CN_mictarget(i,m,imick)
				IF(np_need.GT.zero) THEN
					frac_litt_cue(i,m,k,imick,l)=MAX(cuemin_som, MIN(frac_litt_cue(i,m,k,imick,l),np_supply/np_need))
				ENDIF
			!	(4) MICk - P limit
				np_supply= qd_mic(i,imick,icarbon)*litter_density(i,k,m,l,iphosphorus)/litter_density(i,k,m,l,icarbon)+ &
						   MAX(frac_soil_np*(soilP_density(i,m)-min_soil_mineralP),zero)
				np_need = qd_mic(i,imick,icarbon)/CP_mictarget(i,m,imick)
				IF(np_need.GT.zero) THEN
					frac_litt_cue(i,m,k,imick,l)=MAX(cuemin_som, MIN(frac_litt_cue(i,m,k,imick,l),np_supply/np_need))
				ENDIF
			ENDIF
		  ENDDO
		  
		  WHERE (litter_density(:,k,m,l,icarbon).GT.zero)
		    qd(:,icarbon) = qd_mic(:,imicr,icarbon) + qd_mic(:,imick,icarbon)
		    qd(:,initrogen) = qd(:,icarbon)*litter_density(:,k,m,l,initrogen)/litter_density(:,k,m,l,icarbon)
		    qd(:,iphosphorus) = qd(:,icarbon)*litter_density(:,k,m,l,iphosphorus)/litter_density(:,k,m,l,icarbon)
		    !
		    cflux_litt2mic(:,imicr,m,l)=cflux_litt2mic(:,imicr,m,l) + &
				frac_litt_cue(:,m,k,imicr,l) * qd_mic(:,imicr,icarbon)
		    cflux_litt2mic(:,imick,m,l)=cflux_litt2mic(:,imick,m,l) + &
				frac_litt_cue(:,m,k,imick,l) * qd_mic(:,imick,icarbon)
		  ENDWHERE
		  !
		!  IF(m.EQ.7) THEN
		!	WRITE (numout,*) 'ZHC4_stomatelitter'
		!	WRITE (numout,*) 'ipft',m,'isoilly',l,'ilitt',k
		!	WRITE (numout,*) 'litter_density',litter_density(1,k,m,l,icarbon),'microbe_active',microbe_active(1,:,m,l,icarbon)
		!	WRITE (numout,*) 'control_tempvmax',control_tempvmax(1,l),'vmax_mic',vmax_mic(k,:,l),'vmax_lignin',vmax_lignin(1)
		!	WRITE (numout,*) 'control_tempkm',control_tempkm(1,k,l),'khalf_mic',khalf_mic(k,:,l)
		!	WRITE (numout,*) 'qd',qd(1,icarbon),'qd_mic',qd_mic(1,:,icarbon)
		!	WRITE (numout,*) 'cflux_litt2mic',cflux_litt2mic(1,:,m,l)
		!  ENDIF
		  !
		  !5.4.2 Decomposition of dead leaves
		  IF (k.LE.istructural.AND.l==iabove) THEN
			WHERE (litter_density(:,k,m,l,icarbon).GT.zero)
				dead_leaves(:,m,k) = dead_leaves(:,m,k)*(un - qd(:,icarbon)/litter_density(:,k,m,l,icarbon))
			ENDWHERE
			! decompose same fraction of structural/metabolic part of dead leaves. Note that the
			! the unit for dead_leaves is still g C m^-2
		  ENDIF
		  !
		  !5.4.3 Respiration rate contributed by litter decomposition (g C m^-2)
		  !      And updates of the litter pools
		  IF (l==iabove) THEN
			resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
                ((1. - frac_litt_cue(:,m,k,imicr,l)) * qd_mic(:,imicr,icarbon) + &
				(1. - frac_litt_cue(:,m,k,imick,l))*qd_mic(:,imick,icarbon))*depth_litter(:,m)*10.0			! mg cm-3 --> g m-2
			litt_decay(:,m)=litt_decay(:,m)+(qd_mic(:,imicr,icarbon)+qd_mic(:,imick,icarbon))*depth_litter(:,m)*10.0																							   
			! Minrealization of N, P from litter decomposition
			n_mineralisation(:,m) = n_mineralisation(:,m) + qd(:,initrogen)*depth_litter(:,m)*10.0        ! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) + qd(:,iphosphorus)*depth_litter(:,m)*10.0      ! mg cm-3 --> g m-2
			!
			DO iem = 1,nelements
				litter(:,k,m,l,iem)=litter(:,k,m,l,iem)-qd(:,iem)*depth_litter(:,m)*10.0						! mg cm-3 --> g m-2
			ENDDO
			cflux_litt(:,k,m,l)=qd(:,icarbon)*depth_litter(:,m)*10.0											   
			  !
		  ELSE
			resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
                ((1. - frac_litt_cue(:,m,k,imicr,l)) * qd_mic(:,imicr,icarbon) + &
				(1. - frac_litt_cue(:,m,k,imick,l))*qd_mic(:,imick,icarbon))*(100.0*z_mineralsoil)*10.0		! mg cm-3 --> g m-2
			litt_decay(:,m)=litt_decay(:,m)+(qd_mic(:,imicr,icarbon)+qd_mic(:,imick,icarbon))*(100.0*z_mineralsoil)*10.0																								   
			! Minrealization of N, P from litter decomposition
		    n_mineralisation(:,m) = n_mineralisation(:,m) + qd(:,initrogen)*(100.0*z_mineralsoil)*10.0    ! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) + qd(:,iphosphorus)*(100.0*z_mineralsoil)*10.0  ! mg cm-3 --> g m-2
			  !
			DO iem = 1,nelements
				litter(:,k,m,l,iem)=litter(:,k,m,l,iem)-qd(:,iem)*(100.0*z_mineralsoil)*10.0					! mg cm-3 --> g m-2
			ENDDO
			cflux_litt(:,k,m,l)=qd(:,icarbon)*(100.0*z_mineralsoil)*10.0													 
			  !
		  ENDIF
		   !		   
		ENDDO ! DO k = 1, nlitt
		
		! 5.4.4 Turnover of microbial biomass (mg C cm^-3)
		kk_mic(:,:,:,:,:)=zero
		!tau_mic(:,m,imicr,l)=0.00052*coef_micrtau*EXP(0.3*frac_metabave(:,m,l))
		!tau_mic(:,m,imick,l)=0.00024*coef_micktau*EXP(0.1*frac_metabave(:,m,l))
		tau_mic(:,m,imicr,l)=0.00052*coef_micrtau*EXP(0.3*frac_metabave(:,m,l))*control_tempmictau(:,l) !*MAX(0.8,MIN(1.2,(0.01*npp_longterm(:,m))**0.5))
		tau_mic(:,m,imick,l)=0.00024*coef_micktau*EXP(0.1*frac_metabave(:,m,l))*control_tempmictau(:,l) ! *MAX(0.8,MIN(1.2,(0.01*npp_longterm(:,m))**0.5))

		!kk_mic(:,m,:,l,icarbon) = tau_mic(:,m,:,l)*((microbe_density(:,:,m,l,icarbon))**1.0)
		kk_mic(:,m,imicr,l,icarbon) = tau_mic(:,m,imicr,l)*(MAX((microbe_active(:,imicr,m,l,icarbon))**micr_density_coef, &
									microbe_active(:,imicr,m,l,icarbon)*min_mictau) + &
									taumic_dormant*MAX((microbe_dormant(:,imicr,m,l,icarbon))**micr_density_coef, &
									microbe_dormant(:,imicr,m,l,icarbon)*min_mictau))
		kk_mic(:,m,imick,l,icarbon) = tau_mic(:,m,imick,l)*(MAX((microbe_active(:,imick,m,l,icarbon))**mick_density_coef, &
									microbe_active(:,imick,m,l,icarbon)*min_mictau) + &
									taumic_dormant*MAX((microbe_dormant(:,imick,m,l,icarbon))**mick_density_coef, &
									microbe_dormant(:,imick,m,l,icarbon)*min_mictau))
		!
		kk_mic(:,m,:,l,initrogen) = kk_mic(:,m,:,l,icarbon)/CN_microbe(:,m,:,l)
		kk_mic(:,m,:,l,iphosphorus) = kk_mic(:,m,:,l,icarbon)/CP_microbe(:,m,:,l)
		
		! Minrealization of N, P from microbial turnover
		IF (l==iabove) THEN
			n_mineralisation(:,m) = n_mineralisation(:,m) + &
				(kk_mic(:,m,imicr,l,initrogen) + kk_mic(:,m,imick,l,initrogen))*depth_litter(:,m)*10.0        ! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) + &
				(kk_mic(:,m,imicr,l,iphosphorus)+ kk_mic(:,m,imick,l,iphosphorus))*depth_litter(:,m)*10.0	 ! mg cm-3 --> g m-2
		ELSE
			n_mineralisation(:,m) = n_mineralisation(:,m) + &
				(kk_mic(:,m,imicr,l,initrogen)+kk_mic(:,m,imick,l,initrogen))*(100.0*z_mineralsoil)*10.0    ! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) + &
				(kk_mic(:,m,imicr,l,iphosphorus)+kk_mic(:,m,imick,l,iphosphorus))*(100.0*z_mineralsoil)*10.0 ! mg cm-3 --> g m-2
		ENDIF
		!
		! 5.4.5 Update of microbial pool & immoblization of N,P to microbes
	!	IF(m.EQ.7) THEN		
	!		WRITE (numout,*) 'ZHC4.2_micC',microbe(1,:,m,l,icarbon)
	!		WRITE (numout,*) 'ZHC4.2_micN',microbe(1,:,m,l,initrogen)
	!		WRITE (numout,*) 'ZHC4.2_micP',microbe(1,:,m,l,iphosphorus)
	!		WRITE (numout,*) 'n_mineralisation',n_mineralisation(1,m)
	!		WRITE (numout,*) 'p_mineralisation',p_mineralisation(1,m)
	!		WRITE (numout,*) 'depth_litter',depth_litter(1,m)
	!		WRITE (numout,*) 'kk_mic',kk_mic(1,m,:,l,icarbon)
	!		WRITE (numout,*) 'kk_mic',kk_mic(1,m,:,l,initrogen)
	!		WRITE (numout,*) 'kk_mic',kk_mic(1,m,:,l,iphosphorus)
	!		WRITE (numout,*) 'CNP', CN_microbe(1,m,:),CP_microbe(1,m,:)
	!		WRITE (numout,*) 'litter_density',litter_density(1,:,m,l,icarbon)
	!		WRITE (numout,*) 'tau_mic',tau_mic(1,m,:,l)
	!	ENDIF
		!WRITE (numout,*) 'ZHC4.3_cflux',cflux_litt2mic(1,imicr,m,l)
		!WRITE (numout,*) 'ZHC4.4_depthlitter',depth_litter(1,m)
		!WRITE (numout,*) 'ZHC4.5_kk_mic',kk_mic(1,m,:,l,icarbon)
		!WRITE (numout,*) 'ZHC4.6_CNP',CN_microbe(1,m,:),CP_microbe(1,m,:)
		IF (l==iabove) THEN
			microbe(:,imicr,m,l,icarbon) = microbe(:,imicr,m,l,icarbon)+ &
				(cflux_litt2mic(:,imicr,m,l)-kk_mic(:,m,imicr,l,icarbon))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,icarbon) = microbe(:,imick,m,l,icarbon)+ &
				(cflux_litt2mic(:,imick,m,l)-kk_mic(:,m,imick,l,icarbon))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			microbe(:,imicr,m,l,initrogen) = microbe(:,imicr,m,l,initrogen)+ &
				(cflux_litt2mic(:,imicr,m,l)/CN_mictarget(:,m,imicr)- &
				kk_mic(:,m,imicr,l,icarbon)/CN_microbe(:,m,imicr,l))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,initrogen) = microbe(:,imick,m,l,initrogen)+ &
				(cflux_litt2mic(:,imick,m,l)/CN_mictarget(:,m,imick)- &
				kk_mic(:,m,imick,l,icarbon)/CN_microbe(:,m,imick,l))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			microbe(:,imicr,m,l,iphosphorus) = microbe(:,imicr,m,l,iphosphorus)+ &
				(cflux_litt2mic(:,imicr,m,l)/CP_mictarget(:,m,imicr)- &
				kk_mic(:,m,imicr,l,icarbon)/CP_microbe(:,m,imicr,l))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,iphosphorus) = microbe(:,imick,m,l,iphosphorus)+ &
				(cflux_litt2mic(:,imick,m,l)/CP_mictarget(:,m,imick)- &
				kk_mic(:,m,imick,l,icarbon)/CP_microbe(:,m,imick,l))*depth_litter(:,m)*10.0       		! mg cm-3 --> g m-2
			n_mineralisation(:,m) = n_mineralisation(:,m) - &
				(cflux_litt2mic(:,imicr,m,l)/CN_mictarget(:,m,imicr) + &
				 cflux_litt2mic(:,imick,m,l)/CN_mictarget(:,m,imick)) * depth_litter(:,m)*10.0		 		! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) - &
				(cflux_litt2mic(:,imicr,m,l)/CP_mictarget(:,m,imicr) + &
				 cflux_litt2mic(:,imick,m,l)/CP_mictarget(:,m,imick)) * depth_litter(:,m)*10.0		 		! mg cm-3 --> g m-2
			cflux_mic(:,imicr,m,l)=kk_mic(:,m,imicr,l,icarbon)*depth_litter(:,m)*10.0
			cflux_mic(:,imick,m,l)=kk_mic(:,m,imick,l,icarbon)*depth_litter(:,m)*10.0															
		ELSE
			microbe(:,imicr,m,l,icarbon) = microbe(:,imicr,m,l,icarbon)+ &
				(cflux_litt2mic(:,imicr,m,l)-kk_mic(:,m,imicr,l,icarbon))*(100.0*z_mineralsoil)*10.0   		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,icarbon) = microbe(:,imick,m,l,icarbon)+ &
				(cflux_litt2mic(:,imick,m,l)-kk_mic(:,m,imick,l,icarbon))*(100.0*z_mineralsoil)*10.0   		! mg cm-3 --> g m-2
			microbe(:,imicr,m,l,initrogen) = microbe(:,imicr,m,l,initrogen)+ &
				(cflux_litt2mic(:,imicr,m,l)/CN_mictarget(:,m,imicr)- &
				kk_mic(:,m,imicr,l,icarbon)/CN_microbe(:,m,imicr,l))*(100.0*z_mineralsoil)*10.0        		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,initrogen) = microbe(:,imick,m,l,initrogen)+ &
				(cflux_litt2mic(:,imick,m,l)/CN_mictarget(:,m,imick)- &
				kk_mic(:,m,imick,l,icarbon)/CN_microbe(:,m,imick,l))*(100.0*z_mineralsoil)*10.0     		! mg cm-3 --> g m-2
			microbe(:,imicr,m,l,iphosphorus) = microbe(:,imicr,m,l,iphosphorus)+ &
				(cflux_litt2mic(:,imicr,m,l)/CP_mictarget(:,m,imicr)- &
				kk_mic(:,m,imicr,l,icarbon)/CP_microbe(:,m,imicr,l))*(100.0*z_mineralsoil)*10.0   		! mg cm-3 --> g m-2
			microbe(:,imick,m,l,iphosphorus) = microbe(:,imick,m,l,iphosphorus)+ &
				(cflux_litt2mic(:,imick,m,l)/CP_mictarget(:,m,imick)- &
				kk_mic(:,m,imick,l,icarbon)/CP_microbe(:,m,imick,l))*(100.0*z_mineralsoil)*10.0  
			n_mineralisation(:,m) = n_mineralisation(:,m) - &
				(cflux_litt2mic(:,imicr,m,l)/CN_mictarget(:,m,imicr) + &
				 cflux_litt2mic(:,imick,m,l)/CN_mictarget(:,m,imick)) * (100.0*z_mineralsoil)*10.0	 		! mg cm-3 --> g m-2
			p_mineralisation(:,m) = p_mineralisation(:,m) - &
				(cflux_litt2mic(:,imicr,m,l)/CP_mictarget(:,m,imicr) + &
				 cflux_litt2mic(:,imick,m,l)/CP_mictarget(:,m,imick)) * (100.0*z_mineralsoil)*10.0	 		! mg cm-3 --> g m-2
			cflux_mic(:,imicr,m,l)=kk_mic(:,m,imicr,l,icarbon)*(100.0*z_mineralsoil)*10.0
			cflux_mic(:,imick,m,l)=kk_mic(:,m,imick,l,icarbon)*(100.0*z_mineralsoil)*10.0																
		ENDIF
	!	IF(m.EQ.7) THEN
	!		WRITE (numout,*) 'ZHC4.7_micC',microbe(1,:,m,l,icarbon)
	!		WRITE (numout,*) 'ZHC4.7_micN',microbe(1,:,m,l,initrogen)
	!		WRITE (numout,*) 'ZHC4.7_micP',microbe(1,:,m,l,iphosphorus)
	!		WRITE (numout,*) 'n_mineralisation',n_mineralisation(1,m)
	!		WRITE (numout,*) 'p_mineralisation',p_mineralisation(1,m)
	!	ENDIF
!		
!		WRITE (numout,*) 'ZHC4.8_CN_mic',CN_microbe(1,m,:)
!		WRITE (numout,*) 'ZHC4.9_CNstock_mic',microbe(1,:,m,l,icarbon),'N',microbe(1,:,m,l,initrogen)
		!
		! 5.4.6 C flux from microbial pools to SOM pools
		!
!		WRITE (numout,*) 'ZHC4.10_som_inputC',som_input(1,:,m,icarbon)
!		WRITE (numout,*) 'ZHC4.10_som_inputN',som_input(1,:,m,initrogen)
!		WRITE (numout,*) 'ZHC4.10_som_inputP',som_input(1,:,m,iphosphorus)
		IF (l==iabove) THEN !
			frac_micsom(:,m,imicr,isomp,l) = MIN(1.0, coef_fmicsom1*exp(1.3*clay(:)))
			frac_micsom(:,m,imick,isomp,l) = MIN(1.0, coef_fmicsom2*exp(0.8*clay(:)))
			frac_micsom(:,m,imicr,isomc,l) = MIN(1.0-frac_micsom(:,m,imicr,isomp,l), &
			&								 coef_fmicsom3*exp(-coef_fmicsom5*frac_metabave(:,m,l)))
			frac_micsom(:,m,imick,isomc,l) = MIN(1.0-frac_micsom(:,m,imick,isomp,l), &
			&								 coef_fmicsom4*exp(-coef_fmicsom5*frac_metabave(:,m,l)))
			frac_micsom(:,m,imicr,isoma,l) = un - frac_micsom(:,m,imicr,isomp,l) - &
											 frac_micsom(:,m,imicr,isomc,l)
			frac_micsom(:,m,imick,isoma,l) = un - frac_micsom(:,m,imick,isomp,l) - &
											 frac_micsom(:,m,imick,isomc,l)
			!
			som_input(:,isomp,m,icarbon) = som_input(:,isomp,m,icarbon) + &
							frac_micsom(:,m,imicr,isomp,l)* &
							kk_mic(:,m,imicr,l,icarbon)*depth_litter(:,m)*10.0
			som_input(:,isomp,m,icarbon) = som_input(:,isomp,m,icarbon) + &
							frac_micsom(:,m,imick,isomp,l)* &
							kk_mic(:,m,imick,l,icarbon)*depth_litter(:,m)*10.0
			som_input(:,isomc,m,icarbon) = som_input(:,isomc,m,icarbon) + &
							frac_micsom(:,m,imicr,isomc,l)* &
							kk_mic(:,m,imicr,l,icarbon)*depth_litter(:,m)*10.0
			som_input(:,isomc,m,icarbon) = som_input(:,isomc,m,icarbon) + &
							frac_micsom(:,m,imick,isomc,l)* &
							kk_mic(:,m,imick,l,icarbon)*depth_litter(:,m)*10.0
			som_input(:,isoma,m,icarbon) = som_input(:,isoma,m,icarbon) + &
							frac_micsom(:,m,imicr,isoma,l)* &
							kk_mic(:,m,imicr,l,icarbon)*depth_litter(:,m)*10.0
			som_input(:,isoma,m,icarbon) = som_input(:,isoma,m,icarbon) + &
							frac_micsom(:,m,imick,isoma,l)* &
							kk_mic(:,m,imick,l,icarbon)*depth_litter(:,m)*10.0
			!
!			WRITE (numout,*) 'ZHC4.10_fraction_micrsom_above',frac_micsom(1,m,imicr,:,l), &
!							'ZHC4.10_fraction_micksom_above',frac_micsom(1,m,imick,:,l), &
!							'kk_mic',kk_mic(1,m,:,l,icarbon), 'depth',depth_litter(1,m), &
!							'som_input',som_input(1,:,m,icarbon)
		ELSE
			frac_micsom(:,m,imicr,isomp,l) = MIN(1.0, coef_fmicsom1*0.3*exp(1.3*clay(:)))
			frac_micsom(:,m,imick,isomp,l) = MIN(1.0, coef_fmicsom2*0.2*exp(0.8*clay(:)))
			frac_micsom(:,m,imicr,isomc,l) = MIN(1.0-frac_micsom(:,m,imicr,isomp,l), &
			&								 coef_fmicsom3*exp(-coef_fmicsom5*frac_metabave(:,m,l)))
			frac_micsom(:,m,imick,isomc,l) = MIN(1.0-frac_micsom(:,m,imick,isomp,l), &
			&								 coef_fmicsom4*exp(-coef_fmicsom5*frac_metabave(:,m,l)))
			frac_micsom(:,m,imicr,isoma,l) = un - frac_micsom(:,m,imicr,isomp,l) - &
											 frac_micsom(:,m,imicr,isomc,l)
			frac_micsom(:,m,imick,isoma,l) = un - frac_micsom(:,m,imick,isomp,l) - &
											 frac_micsom(:,m,imick,isomc,l)
			som_input(:,isomp,m,icarbon) = som_input(:,isomp,m,icarbon) + &
							frac_micsom(:,m,imicr,isomp,l)* &
							kk_mic(:,m,imicr,l,icarbon)*(100.0*z_mineralsoil)*10.0
			som_input(:,isomp,m,icarbon) = som_input(:,isomp,m,icarbon) + &
							frac_micsom(:,m,imick,isomp,l)* &
							kk_mic(:,m,imick,l,icarbon)*(100.0*z_mineralsoil)*10.0
			som_input(:,isomc,m,icarbon) = som_input(:,isomc,m,icarbon) + &
							frac_micsom(:,m,imicr,isomc,l)* &
							kk_mic(:,m,imicr,l,icarbon)*(100.0*z_mineralsoil)*10.0
			som_input(:,isomc,m,icarbon) = som_input(:,isomc,m,icarbon) + &
							frac_micsom(:,m,imick,isomc,l)* &
							kk_mic(:,m,imick,l,icarbon)*(100.0*z_mineralsoil)*10.0
			som_input(:,isoma,m,icarbon) = som_input(:,isoma,m,icarbon) + &
							frac_micsom(:,m,imicr,isoma,l)* &
							kk_mic(:,m,imicr,l,icarbon)*(100.0*z_mineralsoil)*10.0
			som_input(:,isoma,m,icarbon) = som_input(:,isoma,m,icarbon) + &
							frac_micsom(:,m,imick,isoma,l)* &
							kk_mic(:,m,imick,l,icarbon)*(100.0*z_mineralsoil)*10.0
			!
!			WRITE (numout,*) 'ZHC4.11_fraction_micrsom_above',frac_micsom(1,m,imicr,:,l), &
!							'ZHC4.11_fraction_micksom_above',frac_micsom(1,m,imick,:,l), &
!							'kk_mic',kk_mic(1,m,:,l,icarbon), 'depth',z_mineralsoil, &
!							'som_input',som_input(1,:,m,icarbon)
		ENDIF
		
	!	IF(m.EQ.7) THEN
	!		WRITE (numout,*) 'ZHC6_stomatelitter'
	!		WRITE (numout,*) 'microbe',microbe(1,:,m,l,icarbon)
	!		WRITE (numout,*) 'frac_micr2socacp',frac_micsom(1,m,imicr,:,l)
	!		WRITE (numout,*) 'frac_mick2socacp',frac_micsom(1,m,imick,:,l)
	!		WRITE (numout,*) 'som_input_C',som_input(1,:,m,icarbon)
	!		WRITE (numout,*) 'som_input_N',som_input(1,:,m,initrogen)
	!		WRITE (numout,*) 'som_input_P',som_input(1,:,m,iphosphorus)
	!		WRITE (numout,*) 'n_mineralisation',n_mineralisation(1,m)
	!		WRITE (numout,*) 'p_mineralisation',p_mineralisation(1,m)
	!	ENDIF
		
		!
	  ENDDO ! DO l = 1, nlevs
	  !
	  !immoblization of N,P to SOM pools
	  DO i = 1,ncarb
		 som_input(:,i,m,initrogen) = som_input(:,i,m,icarbon)/CN_target(:,m,i) ! g N m-2
		 som_input(:,i,m,iphosphorus) = som_input(:,i,m,icarbon)/CP_target(:,m,i) ! g N m-2
		 n_mineralisation(:,m) = n_mineralisation(:,m) - som_input(:,i,m,initrogen) ! g N m-2
		 p_mineralisation(:,m) = p_mineralisation(:,m) - som_input(:,i,m,iphosphorus) ! g N m-2
	  ENDDO
	  
	  WHERE(litt_decay(:,m).GT.zero)
		cue_ave(:,m)=un - resp_hetero_litter(:,m)/litt_decay(:,m)
	  ENDWHERE							  
	  !
	  !IF(m.EQ.7) THEN 
		!	WRITE (numout,*) 'cue_ave',cue_ave(1,2:13)
	  !ENDIF
	  !
	ENDDO ! DO m = 1,nvm
	
	!IF(isbarefallow) THEN
    !    som_input(:,:,:,:)=zero 
	!ENDIF
 ! For calculating turnover times
	CALL xios_orchidee_send_field("Cflux_LITTmet_ab",cflux_litt(:,imetabolic,:,iabove)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_LITTstr_ab",cflux_litt(:,istructural,:,iabove)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_LITTwod_ab",cflux_litt(:,iwoody,:,iabove)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_LITTmet_be",cflux_litt(:,imetabolic,:,ibelow)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_LITTstr_be",cflux_litt(:,istructural,:,ibelow)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_LITTwod_be",cflux_litt(:,iwoody,:,ibelow)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_MICr_ab",cflux_mic(:,imicr,:,iabove)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_MICk_ab",cflux_mic(:,imick,:,iabove)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_MICr_be",cflux_mic(:,imicr,:,ibelow)/dt_sechiba*one_day)
	CALL xios_orchidee_send_field("Cflux_MICk_be",cflux_mic(:,imicr,:,ibelow)/dt_sechiba*one_day)																					  
	CALL xios_orchidee_send_field("MICCUE_ave",cue_ave)													 
	!
	!	WRITE (numout,*) 'ZHC3_stomate_litter: mass check'
	!	DO k=1,3
	!		DO i=1,1
	!			DO m=7,7
	!				WRITE (numout,*) ' element :', k, 'grid', i, 'pft',m
	!				WRITE (numout,*) ' Litter_input_ab', sum(litter_inc(i,:,m,iabove,k))
	!				WRITE (numout,*) ' Litter_input_be', sum(litter_inc(i,:,m,ibelow,k))
	!				WRITE (numout,*) ' Microbes_ab', microbe(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Microbes_be', microbe(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Litter_ab', litter(i,:,m,iabove,k)
	!				WRITE (numout,*) ' Litter_be', litter(i,:,m,ibelow,k)
	!				WRITE (numout,*) ' Repiration', resp_hetero_litter(i,m)
	!				WRITE (numout,*) ' som_input',som_input(i,:,m,k)
	!				WRITE (numout,*) ' Nmin', n_mineralisation(i,m)
	!				WRITE (numout,*) ' Pmin', p_mineralisation(i,m)
	!			ENDDO
	!		ENDDO
	!	ENDDO
	  
!	IF(dsg_debug) THEN 
	!WRITE (numout,*) 'ZHC5_stomatelitter: mass check'
	!DO m=7,7
	!	WRITE (numout,*) 'PFT',m,'litter_carbon',sum(litter(1,:,m,1,icarbon)),sum(litter(1,:,m,2,icarbon)), &
	!		'MicrobialBiomass',sum(microbe(1,:,m,1,icarbon)),sum(microbe(1,:,m,2,icarbon)), &
	!		'SOM_input',sum(som_input(1,:,m,icarbon)),'CN_target',CN_target(1,m,:), &
	!		'CP_target',CP_target(1,m,:)
	!	WRITE (numout,*) 'PFT',m,'litter_N',sum(litter(1,:,m,1,initrogen)),sum(litter(1,:,m,2,initrogen)), &
	!		'MicrobialBiomass',sum(microbe(1,:,m,1,initrogen)),sum(microbe(1,:,m,2,initrogen)), &
	!		'SOM_input',sum(som_input(1,:,m,initrogen)),n_mineralisation(1,m)
	!	WRITE (numout,*) 'PFT',m,'litter_P',sum(litter(1,:,m,1,iphosphorus)),sum(litter(1,:,m,2,iphosphorus)), &
	!		'MicrobialBiomass',sum(microbe(1,:,m,1,iphosphorus)),sum(microbe(1,:,m,2,iphosphorus)), &
	!		'SOM_input',sum(som_input(1,:,m,iphosphorus)),p_mineralisation(1,m)
	!ENDDO
	!    ENDIF
    !
    DO m=1,nvm 
       summ(:,:)=zero 
       DO l = 1, nlevs 
          DO k = 1,nlitt 
             summ(:,:)=summ(:,:)+litter(:,k,m,l,:) 
          ENDDO
       ENDDO
       WHERE(summ(:,icarbon).LE.min_stomate) 
       
          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + & 
               summ(:,icarbon) 
          n_mineralisation(:,m) = n_mineralisation(:,m) + & 
               summ(:,initrogen) 
          p_mineralisation(:,m)   = p_mineralisation(:,m) + &
              summ(:,iphosphorus)

          litter(:,imetabolic ,m,iabove,icarbon)     = zero
          litter(:,istructural,m,iabove,icarbon)     = zero
          litter(:,iwoody     ,m,iabove,icarbon)     = zero
          litter(:,imetabolic ,m,ibelow,icarbon)     = zero
          litter(:,istructural,m,ibelow,icarbon)     = zero
          litter(:,iwoody     ,m,ibelow,icarbon)     = zero

          litter(:,imetabolic ,m,iabove,initrogen)   = zero
          litter(:,istructural,m,iabove,initrogen)   = zero
          litter(:,iwoody     ,m,iabove,initrogen)   = zero
          litter(:,imetabolic ,m,ibelow,initrogen)   = zero
          litter(:,istructural,m,ibelow,initrogen)   = zero
          litter(:,iwoody     ,m,ibelow,initrogen)   = zero

          litter(:,imetabolic ,m,iabove,iphosphorus) = zero
          litter(:,istructural,m,iabove,iphosphorus) = zero
          litter(:,iwoody     ,m,iabove,iphosphorus) = zero
          litter(:,imetabolic ,m,ibelow,iphosphorus) = zero
          litter(:,istructural,m,ibelow,iphosphorus) = zero
          litter(:,iwoody     ,m,ibelow,iphosphorus) = zero
       ENDWHERE
    ENDDO

    ! For YANN
    CALL xios_orchidee_send_field("N_MINERALISATION_LIT",n_mineralisation(:,:)/dt) 	! g N m^-2 day-1
    CALL xios_orchidee_send_field("P_MINERALISATION_LIT",p_mineralisation(:,:)/dt) 	! g P m^-2 day-1
    CALL xios_orchidee_send_field("hetero_resp_lit",resp_hetero_litter(:,:)/dt)		! g C m^-2 day-1

	!! 6. Calculate the potential maximumm amount of physically-protected SOMp
    socphy_max(:,:)=min_stomate
	DO m=1,nvm
		socphy_max(:,m)=MAX(min_stomate,(10.**(phycmax_c1*log10(100*max(clay(:),1.0e-2))+phycmax_c2))* &
		& control_tempdesorp(:,ibelow)*max(bulk(:),500.0)*1.0e-6)     !(som_density(:,isoma,m,icarbon)**phycmax_c3)
	ENDDO						   
  !! 7. calculate fraction of total soil covered by dead leaves

    CALL deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover, &
                   sla_calc)

  !! 8. (Quasi-)Analytical Spin-up : Start filling MatrixA

    IF (spinup_analytic) THEN

       MatrixA(:,:,:,:) = zero
       VectorB(:,:,:) = zero
       
       
       DO m = 1,nvm

          !- MatrixA : carbon fluxes leaving the litter
		  ! Flux leaving the metabolic above litter pool :
		  WHERE (litter_density(:,imetabolic,m,iabove,icarbon).GT.zero)
			MatrixA(:,m,imetabolic_above,imetabolic_above) = - (microbe_density(:,imicr,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(imetabolic,imicr,iabove)/ &
				(control_tempkm(:,imetabolic,iabove)*khalf_mic(imetabolic,imicr,iabove) + litter_density(:,imetabolic,m,iabove,icarbon))+ &
				microbe_density(:,imick,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(imetabolic,imick,iabove)/ &
				(control_tempkm(:,imetabolic,iabove)*khalf_mic(imetabolic,imick,iabove) + litter_density(:,imetabolic,m,iabove,icarbon)))
          ENDWHERE
		  
		  ! Flux leaving the metabolic below litter pool :
		  WHERE (litter_density(:,imetabolic,m,ibelow,icarbon).GT.zero)
			MatrixA(:,m,imetabolic_below,imetabolic_below) = - (microbe_density(:,imicr,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(imetabolic,imicr,ibelow)/ &
				(control_tempkm(:,imetabolic,ibelow)*khalf_mic(imetabolic,imicr,ibelow) + litter_density(:,imetabolic,m,ibelow,icarbon))+ &
				microbe_density(:,imick,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(imetabolic,imick,ibelow)/ &
				(control_tempkm(:,imetabolic,ibelow)*khalf_mic(imetabolic,imick,ibelow) + litter_density(:,imetabolic,m,ibelow,icarbon)))
          ENDWHERE
		  !
		  ! Flux leaving the structural above litter pool :
          WHERE (litter_density(:,istructural,m,iabove,icarbon).GT.zero)
			MatrixA(:,m,istructural_above,istructural_above)= - (microbe_density(:,imicr,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(istructural,imicr,iabove)/ &
				(control_tempkm(:,istructural,iabove)*khalf_mic(istructural,imicr,iabove) + litter_density(:,istructural,m,iabove,icarbon))+ &
				microbe_density(:,imick,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(istructural,imick,iabove)/ &
				(control_tempkm(:,istructural,iabove)*khalf_mic(istructural,imick,iabove) + litter_density(:,istructural,m,iabove,icarbon)))
          ENDWHERE
		  
		  ! Flux leaving the structural below litter pool :
		  WHERE (litter_density(:,istructural,m,ibelow,icarbon).GT.zero)
			MatrixA(:,m,istructural_below,istructural_below) = - (microbe_density(:,imicr,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(istructural,imicr,ibelow)/ &
				(control_tempkm(:,istructural,ibelow)*khalf_mic(istructural,imicr,ibelow) + litter_density(:,istructural,m,ibelow,icarbon))+ &
				microbe_density(:,imick,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(istructural,imick,ibelow)/ &
				(control_tempkm(:,istructural,ibelow)*khalf_mic(istructural,imick,ibelow) + litter_density(:,istructural,m,ibelow,icarbon)))
          ENDWHERE
		  
		  ! Flux leaving the woody above litter pool :
		  WHERE (litter_density(:,iwoody,m,iabove,icarbon).GT.zero)
			MatrixA(:, m, iwoody_above, iwoody_above) = - (microbe_density(:,imicr,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(iwoody,imicr,iabove)/ &
				(control_tempkm(:,iwoody,iabove)*khalf_mic(iwoody,imicr,iabove) + litter_density(:,iwoody,m,iabove,icarbon))+ &
				microbe_density(:,imick,m,iabove,icarbon)* &
				control_tempvmax(:,iabove)*vmax_mic(iwoody,imick,iabove)/ &
				(control_tempkm(:,iwoody,iabove)*khalf_mic(iwoody,imick,iabove) + litter_density(:,iwoody,m,iabove,icarbon)))
		  ENDWHERE
		  
		  ! Flux leaving the woody below litter pool :
		  WHERE (litter_density(:,iwoody,m,ibelow,icarbon).GT.zero)
			MatrixA(:, m, iwoody_below, iwoody_below) = - (microbe_density(:,imicr,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(iwoody,imicr,ibelow)/ &
				(control_tempkm(:,iwoody,ibelow)*khalf_mic(iwoody,imicr,ibelow) + litter_density(:,iwoody,m,ibelow,icarbon))+ &
				microbe_density(:,imick,m,ibelow,icarbon)* &
				control_tempvmax(:,ibelow)*vmax_mic(iwoody,imick,ibelow)/ &
				(control_tempkm(:,iwoody,ibelow)*khalf_mic(iwoody,imick,ibelow) + litter_density(:,iwoody,m,ibelow,icarbon)))
		  ENDWHERE
		  !! imicr-above
          ! Flux received by the imicr from the metabolic above litter pool :
		  WHERE (litter_density(:,imetabolic,m,iabove,icarbon).GT.zero)
          MatrixA(:, m, imicr_above, imetabolic_above) = frac_litt_cue(:,m,imetabolic,imicr,iabove) * &
				microbe_density(:,imicr,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(imetabolic,imicr,iabove)/ &
				(control_tempkm(:,imetabolic,iabove)*khalf_mic(imetabolic,imicr,iabove) + litter_density(:,imetabolic,m,iabove,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imicr from the structural above litter pool :
		  WHERE (litter_density(:,istructural,m,iabove,icarbon).GT.zero)
			MatrixA(:, m, imicr_above, istructural_above) = frac_litt_cue(:,m,istructural,imicr,iabove) * &
				microbe_density(:,imicr,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(istructural,imicr,iabove)/ &
				(control_tempkm(:,istructural,iabove)*khalf_mic(istructural,imicr,iabove) + litter_density(:,istructural,m,iabove,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imicr from the woody above litter pool :
		  WHERE (litter_density(:,iwoody,m,iabove,icarbon).GT.zero)
			MatrixA(:, m, imicr_above, iwoody_above) = frac_litt_cue(:,m,iwoody,imicr,iabove) * &
				microbe_density(:,imicr,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(iwoody,imicr,iabove)/ &
				(control_tempkm(:,iwoody,iabove)*khalf_mic(iwoody,imicr,iabove) + litter_density(:,iwoody,m,iabove,icarbon))
		  ENDWHERE
		  !! imick-above
		  ! Flux received by the imick from the metabolic above litter pool :
		  WHERE (litter_density(:,imetabolic,m,iabove,icarbon).GT.zero)
          MatrixA(:, m, imick_above, imetabolic_above) = frac_litt_cue(:,m,imetabolic,imick,iabove) * &
				microbe_density(:,imick,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(imetabolic,imick,iabove)/ &
				(control_tempkm(:,imetabolic,iabove)*khalf_mic(imetabolic,imick,iabove) + litter_density(:,imetabolic,m,iabove,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imick from the structural above litter pool :
		  WHERE (litter_density(:,istructural,m,iabove,icarbon).GT.zero)
			MatrixA(:, m, imick_above, istructural_above) = frac_litt_cue(:,m,istructural,imick,iabove) * &
				microbe_density(:,imick,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(istructural,imick,iabove)/ &
				(control_tempkm(:,istructural,iabove)*khalf_mic(istructural,imick,iabove) + litter_density(:,istructural,m,iabove,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imick from the woody above litter pool :
		  WHERE (litter_density(:,iwoody,m,iabove,icarbon).GT.zero)
			MatrixA(:, m, imick_above, iwoody_above) = frac_litt_cue(:,m,iwoody,imick,iabove) * &
				microbe_density(:,imick,m,iabove,icarbon) * control_tempvmax(:,iabove)*vmax_mic(iwoody,imick,iabove)/ &
				(control_tempkm(:,iwoody,iabove)*khalf_mic(iwoody,imick,iabove) + litter_density(:,iwoody,m,iabove,icarbon))
		  ENDWHERE
		  !! imicr-below
          ! Flux received by the imicr from the metabolic below litter pool :
		  WHERE (litter_density(:,imetabolic,m,ibelow,icarbon).GT.zero)
          MatrixA(:, m, imicr_below, imetabolic_below) = frac_litt_cue(:,m,imetabolic,imicr,ibelow) * &
				microbe_density(:,imicr,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(imetabolic,imicr,ibelow)/ &
				(control_tempkm(:,imetabolic,ibelow)*khalf_mic(imetabolic,imicr,ibelow) + litter_density(:,imetabolic,m,ibelow,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imicr from the structural below litter pool :
		  WHERE (litter_density(:,istructural,m,ibelow,icarbon).GT.zero)
			MatrixA(:, m, imicr_below, istructural_below) = frac_litt_cue(:,m,istructural,imicr,ibelow) * &
				microbe_density(:,imicr,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(istructural,imicr,ibelow)/ &
				(control_tempkm(:,istructural,ibelow)*khalf_mic(istructural,imicr,ibelow) + litter_density(:,istructural,m,ibelow,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imicr from the woody below litter pool :
		  WHERE (litter_density(:,iwoody,m,ibelow,icarbon).GT.zero)
			MatrixA(:, m, imicr_below, iwoody_below) = frac_litt_cue(:,m,iwoody,imicr,ibelow) * &
				microbe_density(:,imicr,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(iwoody,imicr,ibelow)/ &
				(control_tempkm(:,iwoody,ibelow)*khalf_mic(iwoody,imicr,ibelow) + litter_density(:,iwoody,m,ibelow,icarbon))
		  ENDWHERE
		  !! imick-below
		  ! Flux received by the imick from the metabolic below litter pool :
		  WHERE (litter_density(:,imetabolic,m,ibelow,icarbon).GT.zero)
          MatrixA(:, m, imick_below, imetabolic_below) = frac_litt_cue(:,m,imetabolic,imick,ibelow) * &
				microbe_density(:,imick,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(imetabolic,imick,ibelow)/ &
				(control_tempkm(:,imetabolic,ibelow)*khalf_mic(imetabolic,imick,ibelow) + litter_density(:,imetabolic,m,ibelow,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imick from the structural below litter pool :
		  WHERE (litter_density(:,istructural,m,ibelow,icarbon).GT.zero)
			MatrixA(:, m, imick_below, istructural_below) = frac_litt_cue(:,m,istructural,imick,ibelow) * &
				microbe_density(:,imick,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(istructural,imick,ibelow)/ &
				(control_tempkm(:,istructural,ibelow)*khalf_mic(istructural,imick,ibelow) + litter_density(:,istructural,m,ibelow,icarbon))
		  ENDWHERE
		  
		  ! Flux received by the imick from the woody below litter pool :
		  WHERE (litter_density(:,iwoody,m,ibelow,icarbon).GT.zero)
			MatrixA(:, m, imick_below, iwoody_below) = frac_litt_cue(:,m,iwoody,imick,ibelow) * &
				microbe_density(:,imick,m,ibelow,icarbon) * control_tempvmax(:,ibelow)*vmax_mic(iwoody,imick,ibelow)/ &
				(control_tempkm(:,iwoody,ibelow)*khalf_mic(iwoody,imick,ibelow) + litter_density(:,iwoody,m,ibelow,icarbon))
		  ENDWHERE
		  !! Turnover of microbes
		  ! Turnover of imicr_above
		  MatrixA(:, m, imicr_above, imicr_above) = -tau_mic(:,m,imicr,iabove)
		  ! Turnover of imick_above
		  MatrixA(:, m, imick_above, imick_above) = -tau_mic(:,m,imick,iabove)
		  ! Turnover of imicr_below
		  MatrixA(:, m, imicr_below, imicr_below) = -tau_mic(:,m,imicr,ibelow)
		  ! Turnover of imick_below
		  MatrixA(:, m, imick_below, imick_below) = -tau_mic(:,m,imick,ibelow)
		  
          !- VectorB : carbon input -
          WHERE (depth_litter(:,m).GT.zero)
			VectorB(:,m,istructural_above) = 0.1*litter_inc(:,istructural,m,iabove,icarbon)/depth_litter(:,m)
			VectorB(:,m,imetabolic_above)  = 0.1*litter_inc(:,imetabolic,m,iabove,icarbon)/depth_litter(:,m)
			VectorB(:,m,iwoody_above)      = 0.1*litter_inc(:,iwoody,m,iabove,icarbon)/depth_litter(:,m)
		  ENDWHERE
          VectorB(:,m,istructural_below) = 0.1*litter_inc(:,istructural,m,ibelow,icarbon)/(100.0*z_mineralsoil)
          VectorB(:,m,imetabolic_below)  = 0.1*litter_inc(:,imetabolic,m,ibelow,icarbon)/(100.0*z_mineralsoil)
          VectorB(:,m,iwoody_below)      = 0.1*litter_inc(:,iwoody,m,ibelow,icarbon)/(100.0*z_mineralsoil)

          IF (printlev>=4) WRITE(numout,*) 'We filled MatrixA and VectorB' 
         

          ! DSG: there WHERE statements are problematic ; we need to check the
          ! icarbon stocks rather the nutrient stocks as the nutrients stocks << carbon stocks
          ! and we need to have nutrient when we have carbon
          !DSG exchange initrogen 
          !WHERE(litter(:,istructural,m,iabove,initrogen) .GT. min_stomate)
          WHERE(litter(:,istructural,m,iabove,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,istructural_above) = ( CN_som_litter_longterm(:,m,istructural_above) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,iabove,icarbon)/litter(:,istructural,m,iabove,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,istructural_above) = ( CP_som_litter_longterm(:,m,istructural_above) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,iabove,icarbon)/litter(:,istructural,m,iabove,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,istructural_above) = zero
             CP_som_litter_longterm(:,m,istructural_above) = zero
          ENDWHERE
          
          !WHERE(litter(:,istructural,m,ibelow,initrogen) .GT. min_stomate)
          WHERE(litter(:,istructural,m,ibelow,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,istructural_below) = ( CN_som_litter_longterm(:,m,istructural_below) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,ibelow,icarbon)/litter(:,istructural,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,istructural_below) = ( CP_som_litter_longterm(:,m,istructural_below) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,ibelow,icarbon)/litter(:,istructural,m,ibelow,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,istructural_below) = zero
             CP_som_litter_longterm(:,m,istructural_below) = zero
          ENDWHERE

          !WHERE(litter(:,imetabolic,m,iabove,initrogen) .GT. min_stomate)
          WHERE(litter(:,imetabolic,m,iabove,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,imetabolic_above) = ( CN_som_litter_longterm(:,m,imetabolic_above) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,iabove,icarbon)/litter(:,imetabolic,m,iabove,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,imetabolic_above) = ( CP_som_litter_longterm(:,m,imetabolic_above) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,iabove,icarbon)/litter(:,imetabolic,m,iabove,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,imetabolic_above) = zero
             CP_som_litter_longterm(:,m,imetabolic_above) = zero
          ENDWHERE

          !WHERE(litter(:,imetabolic,m,ibelow,initrogen) .GT. min_stomate)
          WHERE(litter(:,imetabolic,m,ibelow,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,imetabolic_below) = ( CN_som_litter_longterm(:,m,imetabolic_below) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,ibelow,icarbon)/litter(:,imetabolic,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,imetabolic_below) = ( CP_som_litter_longterm(:,m,imetabolic_below) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,ibelow,icarbon)/litter(:,imetabolic,m,ibelow,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,imetabolic_below) = zero
             CP_som_litter_longterm(:,m,imetabolic_below) = zero
          ENDWHERE

          !WHERE(litter(:,iwoody,m,iabove,initrogen) .GT. min_stomate)
          WHERE(litter(:,iwoody,m,iabove,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,iwoody_above) = ( CN_som_litter_longterm(:,m,iwoody_above) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,iabove,icarbon)/litter(:,iwoody,m,iabove,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,iwoody_above) = ( CP_som_litter_longterm(:,m,iwoody_above) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,iabove,icarbon)/litter(:,iwoody,m,iabove,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,iwoody_above) = zero
             CP_som_litter_longterm(:,m,iwoody_above) = zero
          ENDWHERE
          
          !WHERE(litter(:,iwoody,m,ibelow,initrogen) .GT. min_stomate)
          WHERE(litter(:,iwoody,m,ibelow,icarbon) .GT. min_stomate)
             CN_som_litter_longterm(:,m,iwoody_below) = ( CN_som_litter_longterm(:,m,iwoody_below) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,ibelow,icarbon)/litter(:,iwoody,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)

             CP_som_litter_longterm(:,m,iwoody_below) = ( CP_som_litter_longterm(:,m,iwoody_below) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,ibelow,icarbon)/litter(:,iwoody,m,ibelow,iphosphorus) * dt)/ (tau_CN_longterm)
          ELSEWHERE
             CN_som_litter_longterm(:,m,iwoody_below) = zero
             CP_som_litter_longterm(:,m,iwoody_below) = zero
          ENDWHERE

       ENDDO ! Loop over # PFTs
 
          
    ENDIF ! spinup analytic 

    IF (printlev>=4) WRITE(numout,*) 'Leaving littercalc'

  END SUBROUTINE littercalc


!! ==============================================================================================================================\n
!! SUBROUTINE   : deadleaf
!!
!>\BRIEF        This routine calculates the deadleafcover. 
!!
!! DESCRIPTION  : It first calculates the lai corresponding to the dead leaves (LAI) using 
!! the dead leaves carbon content (DL) the specific leaf area (sla) and the 
!! maximal coverage fraction of a PFT (vegetmax) using the following equations:
!! \latexonly
!! \input{deadleaf1.tex}
!! \endlatexonly
!! \n
!! Then, the dead leaf cover (DLC) is calculated as following:\n
!! \latexonly
!! \input{deadleaf2.tex}
!! \endlatexonly
!! \n
!! 
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE: ::deadleaf_cover
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover, &
                       sla_calc)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                          :: npts           !! Domain size - number of grid pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(in)  :: dead_leaves    !! Dead leaves per ground unit area, per PFT, 
                                                                          !! metabolic and structural  
                                                                          !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)          :: veget_cov_max  !! PFT "Maximal" coverage fraction of a PFT defined in 
                                                                          !! the input vegetation map 
                                                                          !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)          :: sla_calc       !! leaf age-related SLA
    
    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts), INTENT(out)           :: deadleaf_cover !! Fraction of soil covered by dead leaves over all PFTs
                                                                          !! (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts)                        :: dead_lai       !! LAI of dead leaves @tex $(m^2 m^{-2})$ @endtex
    INTEGER(i_std)                                      :: j              !! Index (unitless)
!_ ================================================================================================================================
    
  !! 1. LAI of dead leaves
  
    dead_lai(:) = zero

    DO j = 1,nvm !Loop over PFTs
       dead_lai(:) = dead_lai(:) + ( dead_leaves(:,j,imetabolic) + &
                     dead_leaves(:,j,istructural) ) * sla_calc(:,j) &
            * veget_cov_max(:,j)
    ENDDO

  !! 2. fraction of soil covered by dead leaves

    deadleaf_cover(:) = un - exp( - 0.5 * dead_lai(:) )

    IF (printlev>=4) WRITE(numout,*) 'Leaving deadleaf'

  END SUBROUTINE deadleaf


!! ================================================================================================================================
!! FUNCTION     : control_moistvmax_func
!!
!>\BRIEF        Calculate moisture control for litter and soil C decomposition
!!
!! DESCRIPTION  : Calculate moisture control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_moist_func1.tex}
!! \endlatexonly
!! \n
!! with M the moisture control factor and soilmoisutre, the soil moisture 
!! calculated in sechiba.
!! Then, the function is ranged between Moistcont_min and 1:\n
!! \latexonly
!! \input{control_moist_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE : ::moistfunc_result
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
FUNCTION control_moistvmax_func (npts, moist_in, bulk, clay) RESULT (moistfunc_result)

  !! 0. Variable and parameter declaration
    REAL(r_std), PARAMETER                   :: soildens = 2650.0   !! Soil density [kg/m**3]   
    REAL(r_std), PARAMETER                   :: bo2 = 0.75          !! O2 supply restriction factor
	REAL(r_std), PARAMETER                   :: ns = 2.0            !! Saturation exponent
	REAL(r_std), PARAMETER                   :: k_theta = 0.1       !! Moisture constant
	REAL(r_std), PARAMETER                   :: opt_moist = 0.65    !! optimatic moisture constant
    !! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)               :: npts                !! Domain size - number of grid pixel (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_in            !! relative humidity (unitless)
	REAL(r_std), DIMENSION(npts), INTENT(in) :: bulk                !! bulk density (kg/m**3)
	REAL(r_std), DIMENSION(npts), INTENT(in) :: clay                !! soil clay fraction (unitless, 0-1)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)             :: moistfunc_result    !! Moisture control factor (0.25-1, unitless)
	REAL(r_std), DIMENSION(npts)             :: op_swc              !! Optimum water content	
    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(npts)             :: a_fact               !! SOC-microorganism collocation factor
	INTEGER(i_std)                           :: ig                   !! Index (unitless)
	!REAL(r_std), DIMENSION(npts)             :: poros               !! Soil porosity
	!REAL(r_std), DIMENSION(npts)             :: moist               !! Relative humidity (unitless, 0-1)
!_ ================================================================================================================================
	!! Old function	
    !moistfunc_result(:) = -moist_coeff(1) * moist_in(:) * moist_in(:) + moist_coeff(2)* moist_in(:) - moist_coeff(3)
    !moistfunc_result(:) = MAX( moistcont_min, MIN( un, moistfunc_result(:) ) )
	
	!! New function introduced by H. Zhang. 
	!!Reference: Yan et al., 2019 Nature Communicatins, A moisture function of soil heterotrophic respiration that incorporates
	!!           microscale processes
	a_fact(:)=zero
	moistfunc_result(:)=zero
    !op_swc(:)=zero
	!poros(:)=zero
	!moist(:)=zero
	
	WHERE(clay(:).LE.0.016)
	    a_fact(:)=zero
    ELSEWHERE ((clay(:).GT.0.016).AND.(clay(:).LE.0.37))
	    a_fact(:)=MAX(zero,2.8*clay(:)-0.046)
	ELSEWHERE (clay(:).GT.0.37)
	    a_fact(:)=1.0
	ENDWHERE
	
    !poros(:) = un-(bulk(:)/soildens)
	!op_swc(:)= 0.65*poros(:)
	!moist(:) = moist_in(:)*poros(:)
	DO ig=1,npts
	
	    IF (moist_in(ig).LE.opt_moist) THEN
			! Default
	        moistfunc_result(ig)=MAX(moistcont_min, &
			&	((k_theta+opt_moist)/(k_theta+moist_in(ig)))*((moist_in(ig)/opt_moist)**(1.0+a_fact(ig)*ns)))
			!moistfunc_result(ig)=MAX(moistcont_min, &
			!&	((k_theta+opt_moist)/(k_theta+moist_in(ig)))*((moist_in(ig)/opt_moist)**(3.0+clay(ig))))
	    ELSEIF (moist_in(ig).GT.opt_moist.AND.(1.0-moist_in(ig)).GT.zero) THEN
	        moistfunc_result(ig)= MAX(moistcont_min,((1.0-moist_in(ig))/(1.0-opt_moist))**bo2)
		ELSE
		    moistfunc_result(ig)=un
	    ENDIF
		moistfunc_result(ig) = MIN(un, moistfunc_result(ig))
	    !WRITE (numout,*) 'ZHC_stomatelitter2'
	    !WRITE (numout,*) 'a_fact',a_fact(ig)
	    !WRITE (numout,*) 'moist_in4',moist_in(ig)
	    !WRITE (numout,*) 'moistfunc_result5',moistfunc_result(ig)
	ENDDO !DO i=1,npts

  END FUNCTION control_moistvmax_func
!! ================================================================================================================================																															   
!! FUNCTION     : control_moist_func
!!
!>\BRIEF        Calculate moisture control for litter and soil C decomposition
!!
!! DESCRIPTION  : Calculate moisture control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_moist_func1.tex}
!! \endlatexonly
!! \n
!! with M the moisture control factor and soilmoisutre, the soil moisture 
!! calculated in sechiba.
!! Then, the function is ranged between Moistcont_min and 1:\n
!! \latexonly
!! \input{control_moist_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE : ::moistfunc_result
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
  
  FUNCTION control_moist_func (npts, moist_in) RESULT (moistfunc_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
          
    INTEGER(i_std), INTENT(in)               :: npts                !! Domain size - number of grid pixel (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_in            !! relative humidity (unitless)

    !! 0.2 Output variables
   
    REAL(r_std), DIMENSION(npts)             :: moistfunc_result    !! Moisture control factor (0.25-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    moistfunc_result(:) = -moist_coeff(1) * moist_in(:) * moist_in(:) + moist_coeff(2)* moist_in(:) - moist_coeff(3)
    moistfunc_result(:) = MAX( moistcont_min, MIN( un, moistfunc_result(:) ) )

  END FUNCTION control_moist_func

!! ================================================================================================================================
!! FUNCTION     : control_moistau_func
!!
!>\BRIEF        Calculate moisture control for microbial turnover
!!
!! DESCRIPTION  : Calculate moisture control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_moist_func1.tex}
!! \endlatexonly
!! \n
!! with M the moisture control factor and soilmoisutre, the soil moisture 
!! calculated in sechiba.
!! Then, the function is ranged between Moistcont_min and 1:\n
!! \latexonly
!! \input{control_moist_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE : ::moistfunc_result
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
  
  FUNCTION control_moistau_func (npts, moist_in, moist_longterm) RESULT (moistfunctau_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
          
    INTEGER(i_std), INTENT(in)               :: npts                !! Domain size - number of grid pixel (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_in            !! relative humidity (unitless)
	REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_longterm      !! relative humidity (unitless)

    !! 0.2 Output variables
   
    REAL(r_std), DIMENSION(npts)             :: moistfunctau_result    !! Moisture control factor (0.25-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    moistfunctau_result(:) = exp(coef_moitau1 * MIN(zero,(moist_in(:) - moist_longterm(:))) - coef_moitau2)

  END FUNCTION control_moistau_func
!! ================================================================================================================================
!! FUNCTION     : control_temp_func
!!
!>\BRIEF        Calculate temperature control for litter and soild C decomposition
!!
!! DESCRIPTION  : Calculate temperature control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_temp_func1.tex}
!! \endlatexonly
!! \n
!! with T the temperature control factor, temp the temperature in Kelvin of 
!! the air (for aboveground litter) or of the soil (for belowground litter 
!! and soil)
!! Then, the function is limited in its maximal range to 1:\n
!! \latexonly
!! \input{control_temp_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE: ::tempfunc_result
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  FUNCTION control_temp_func (npts, temp_in) RESULT (tempfunc_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunc_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    tempfunc_result(:) = exp( soil_Q10 * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
    tempfunc_result(:) = MIN( un, tempfunc_result(:) )

  END FUNCTION control_temp_func
  !
  !! ================================================================================================================================
!! FUNCTION     : control_tempvmax_func
!!
!>\BRIEF        Calculate temperature control for microbial maximum reaction velocity
!!              Ref: Wieder et al., 2015
!! \n
!_ ================================================================================================================================

  FUNCTION control_tempvmax_func (npts, temp_in) RESULT (tempfuncvmax_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfuncvmax_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

!    tempfunc_result(:) = exp( soil_Q10 * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
!    tempfunc_result(:) = MIN( un, tempfunc_result(:) )
	tempfuncvmax_result(:) = EXP(slp_vmax_tem*(temp_in(:) - ZeroCelsius) + int_vmax_tem)

  END FUNCTION control_tempvmax_func
  !
  !! ================================================================================================================================
!! FUNCTION     : control_tempvkm_func
!!
!>\BRIEF        Calculate temperature control for microbial half-saturation constant
!!              Ref: Wieder et al., 2015
!! \n
!_ ================================================================================================================================

  FUNCTION control_tempkm_func (npts, temp_in) RESULT (tempfunckm_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts,nlitt)         :: tempfunckm_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

!    tempfunc_result(:) = exp( soil_Q10 * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
!    tempfunc_result(:) = MIN( un, tempfunc_result(:) )
	tempfunckm_result(:,imetabolic) = exp(slp1_khalf_tem *(temp_in(:) - ZeroCelsius) + int_khalf_tem)
	tempfunckm_result(:,istructural) = exp(slp2_khalf_tem *(temp_in(:) - ZeroCelsius) + int_khalf_tem)
	tempfunckm_result(:,iwoody) = exp(slp2_khalf_tem *(temp_in(:) - ZeroCelsius) + int_khalf_tem)

  END FUNCTION control_tempkm_func
  !
  !! ================================================================================================================================
!! FUNCTION     : control_temptau_func
!!
!>\BRIEF        Calculate temperature control for active microbe
!! \n
!_ ================================================================================================================================

  FUNCTION control_temptau_func (npts, temp_in, temp_longterm) RESULT (tempfunctau_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)
	REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_longterm   !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunctau_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

	tempfunctau_result(:) = exp(coef_temtau1 * MIN(zero,(temp_in(:) - temp_longterm(:))) - coef_temtau2)
	!tempfunctau_result(:) = 1.0-1.0/(1.0+exp(coef_temtau1 * (temp_in(:) - temp_longterm(:)) - coef_temtau2))

  END FUNCTION control_temptau_func
  
	!_ ================================================================================================================================
  !! FUNCTION     : control_tempmictau_func
!!
!>\BRIEF        Calculate temperature control for microbial turnover rate
!! Function is obtained based on Wang et al., 2018, Elevated temperature increases the accumulation of microbial necromass 
!! nitrogen in soil via increasing microbial turnover. Global Change Biology, 26, 5277-5289.
!! \n
!_ ================================================================================================================================

  FUNCTION control_tempmictau_func (npts, temp_in, temp_longterm) RESULT (tempfunctau_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)
	REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_longterm   !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunctau_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

	tempfunctau_result(:) = MIN(un,coef_temmictau1*exp(coef_temmictau2*(temp_longterm(:)- ZeroCelsius)))
	!tempfunctau_result(:) = 1.0-1.0/(1.0+exp(coef_temtau1 * (temp_in(:) - temp_longterm(:)) - coef_temtau2))

  END FUNCTION control_tempmictau_func
!! FUNCTION     : control_tempdesrp_func
!!
!>\BRIEF        Calculate temperature control for DOC desorption
!! \n
!_ ================================================================================================================================

  FUNCTION control_tempdesorp_func (npts, temp_in) RESULT (tempfuncdesorp_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfuncdesorp_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

	!tempfuncdesorp_result(:) = min(100.0,exp(coef_tempdesorp * (temp_in(:) - (10.0+ZeroCelsius))))
	tempfuncdesorp_result(:) = phycmax_c3/exp(coef_tempdesorp * (temp_in(:) - (10.0+ZeroCelsius)))
	!tempfunctau_result(:) = 1.0-1.0/(1.0+exp(coef_temtau1 * (temp_in(:) - temp_longterm(:)) - coef_temtau2))

  END FUNCTION control_tempdesorp_func
!_ ================================================================================================================================

END MODULE stomate_litter

